from PyTASTileSystem import *
from grid_generator import GetInteriorTiles
import math
import sys
import getopt
import copy


color_aqua = 'rgb(0,255,255)'
color_yellow = 'rgb(255,255,0)'
color_pink = 'rgb(255,0,255)'
color_grey = 'rgb(128,128,128)'
color_purple = 'rgb(127,0,255)'
color_red = 'rgb(255,0,0)'

color_map = {'00B': 'Red', '01B': 'Green', '10B': 'Black', '11B': 'White',
             '00I': color_pink, '01I': 'Blue', '10I': color_yellow, '11I': color_aqua}


def DisplayHelp():
    print("\nThis script takes as input a pattern and a row width for the pattern plus blanks, and possibly the")
    print("names of output files, and outputs files for an aTAM system that copies that pattern across a row")
    print("with that width. If the option is used for 3D, the aTAM system will generate a second layer that")
    print("forms a grid from the pattern.")
    print("\nUsage:")
    print("-h, --help\t\t\t\t\tDisplays this help message.")
    print("-p <pattern>, --pattern=<pattern>\t\t[Optional] String representing the pattern to be repeatedly")
    print("\t\t\t\t\t\t output. Default is '01101011'")
    print("-w <width>, --width=<width>\t\t\t[Optional] The total width of the starting row (which will be extended")
    print("\t\t\t\t\t\t with blanks if the value given is greater than the pattern length). If not specified,")
    print("\t\t\t\t\t\tthe length of the pattern + 10 will be used.")
    print("-o <out file name>, --out=<out file name>\t[Optional] Base name of output files. If not specified,")
    print("\t\t\t\t\t\t`pattern{pattern}_width{width}.tdp' will be used.")
    print("-d <dir>, --dir=<dir>\t\t\t\t[Optional] Directory for output files. If not specified,")
    print("\t\t\t\t\t\tthe current directory will be used.")
    print("-f, --fill\t\t\t\t\t[Optional] If this option is specified, the filler tile will be included.")
    print("-t, --three\t\t\t\t\t[Optional] If this option is specified, 3D tiles will be generated.")
    print("")
    exit()


def ColorMapper(row_bit, col_bit, row_mark, col_mark):    
    if ('B' in row_mark) or ('B' in col_mark):
        mark = 'B'
    else:
        mark = 'I'
    color = color_map[f'{row_bit}{col_bit}{mark}']
    return color

def GetExtenderInitialRowTiles(pattern, color_mapper, chunk_len, diagonal_signal):
    tile_set = dict()
    tile_list = []

    row = None
    for i in range(0, len(pattern), chunk_len):
        tile_type_name = f'Init{i}'

        curr = pattern[i:i+chunk_len]
        tile_type_label = f'{curr}'
        t = PyTASTileType(name=tile_type_name, label=tile_type_label)

        if i == 0:
            t.sides[N].label = 'L' + curr + diagonal_signal
            t.sides[N].strength = 2

            t.sides[E].label = tile_type_name
            t.sides[E].strength = 2
            
        elif i == (len(pattern)-chunk_len):
            t.sides[N].label = f'{curr}R'
            t.sides[N].strength = 1

            t.sides[W].label = f'Init{i-chunk_len}'
            t.sides[W].strength = 2

            t.sides[E].label = f'fill'
            t.sides[E].strength = 1
        else:
            t.sides[N].label = curr
            t.sides[N].strength = 1

            t.sides[W].label = f'Init{i-chunk_len}'
            t.sides[W].strength = 2

            t.sides[E].label = tile_type_name
            t.sides[E].strength = 2
        
        if curr[0] == '_':
            t.color = color_yellow
        elif curr[0] == '0':
            t.color = 'Black'
        else:
            t.color = 'White'
        tile_set[tile_type_name] = t
        tile_list.append(t)
    return tile_set, tile_list

def GetSecondPlaneFirstRowTiles(color_mapper, diagonal_signal):
    tile_set = dict()

    for row in [0,1]:
        for col in [0,1]:
            for col_mark in ['B', 'I']:
                # Create tiles for rightmost column to start the row
                tile_type_name = f'Z1RtoL1_{row}_{col}{col_mark}'
                tile_type_label = tile_type_name
                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=True)
                t.sides[D].label = f'{row}_{col}{col_mark}R'
                t.sides[D].strength = 2
                t.sides[S].label = f'{col}{col_mark}'
                t.sides[S].strength = 1
                t.sides[W].label = f'<{row}'
                t.sides[W].strength = 1
                t.color = color_mapper(row, col, 'B', col_mark)
                tile_set[tile_type_name] = t

                tile_type_name = f'Z1RtoL2_{row}_{col}{col_mark}'
                tile_type_label = tile_type_name
                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=True)
                t.sides[D].label = f'{col}{col_mark}'
                t.sides[D].strength = 1
                t.sides[S].label = f'{col}{col_mark}'
                t.sides[S].strength = 1
                t.sides[E].label = f'<{row}'
                t.sides[E].strength = 1
                t.sides[W].label = f'<{row}'
                t.sides[W].strength = 1
                t.color = color_mapper(row, col, 'B', col_mark)
                tile_set[tile_type_name] = t

                tile_type_name = f'Z1RtoL2*_{row}_{col}{col_mark}'
                tile_type_label = tile_type_name
                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=True)
                t.sides[D].label = f'{col}{col_mark}{diagonal_signal}'
                t.sides[D].strength = 1
                t.sides[S].label = f'{col}{col_mark}{diagonal_signal}'
                t.sides[S].strength = 2
                t.sides[E].label = f'<{row}'
                t.sides[E].strength = 1
                t.sides[W].label = f'<{row}'
                t.sides[W].strength = 1
                t.color = color_mapper(row, col, 'B', col_mark)
                tile_set[tile_type_name] = t

    return tile_set
            

def GetExtenderTiles(color_mapper, diagonal_signal, top_diagonal_signal, b_3D):
    tile_set = dict()

    ##########################################################################################
    #### Left-to-right tiles
    for col in [0,1]:
        for col_mark in ['B', 'I']:

            # Create tiles for leftmost column, growing right-to-left
            tile_type_name = f'LtoR1_{col}_{col}{col_mark}{diagonal_signal}'
            tile_type_label = tile_type_name
            t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
            t.sides[N].label = f'L{col}{col_mark}'
            t.sides[N].strength = 1
            t.sides[E].label = f'{col}>{col}{col_mark}{diagonal_signal}'
            t.sides[E].strength = 1
            t.sides[S].label = f'L{col}{col_mark}{diagonal_signal}'
            t.sides[S].strength = 2
            if b_3D:
                t.sides[U].label = f'{col}{col_mark}'
                t.sides[U].strength = 1
            t.color = color_mapper(1, col, 'B', col_mark)
            tile_set[tile_type_name] = t

            tile_type_name = f'LtoR1_{col}_{col}{col_mark}'
            tile_type_label = tile_type_name
            t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
            t.sides[N].label = f'L{col}{col_mark}'
            t.sides[N].strength = 1
            t.sides[E].label = f'{col}{diagonal_signal}>'
            t.sides[E].strength = 1
            t.sides[S].label = f'L{col}{col_mark}'
            t.sides[S].strength = 2
            if b_3D:
                t.sides[U].label = f'{col}{col_mark}'
                t.sides[U].strength = 1
            t.color = color_mapper(1, col, 'B', col_mark)
            tile_set[tile_type_name] = t

            for lcol in [0,1]:  # Copy the leftmost bit through the entire row
                
                # Create the tiles to the left of the diagonal signal (i.e. they've already been copied)
                tile_type_name = f'LtoR10_{lcol}_{col}{col_mark}'
                tile_type_label = tile_type_name
                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                t.sides[N].label = f'{col}{col_mark}'
                t.sides[N].strength = 1
                t.sides[E].label = f'{lcol}>'
                t.sides[E].strength = 1
                t.sides[S].label = f'{col}{col_mark}'
                t.sides[S].strength = 1
                t.sides[W].label = f'{lcol}>'
                t.sides[W].strength = 1
                if b_3D:
                    t.sides[U].label = f'{col}{col_mark}'
                    t.sides[U].strength = 1
                t.color = color_mapper(1, col, 'B', col_mark)
                tile_set[tile_type_name] = t

                # Create the tile for the second column that will have a '*' in the second plane
                tile_type_name = f'LtoR10_{lcol}*_{col}{col_mark}'
                tile_type_label = tile_type_name
                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                t.sides[N].label = f'{col}{col_mark}'
                t.sides[N].strength = 1
                t.sides[E].label = f'{lcol}>'
                t.sides[E].strength = 1
                t.sides[S].label = f'{col}{col_mark}'
                t.sides[S].strength = 1
                t.sides[W].label = f'{lcol}{diagonal_signal}>'
                t.sides[W].strength = 1
                if b_3D:
                    t.sides[U].label = f'{col}{col_mark}{top_diagonal_signal}'
                    t.sides[U].strength = 1
                t.color = color_mapper(1, col, 'B', col_mark)
                tile_set[tile_type_name] = t

                tile_type_name = f'LtoR11_{lcol}_{col}{col_mark}{diagonal_signal}'
                tile_type_label = tile_type_name
                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                t.sides[N].label = f'{col}{col_mark}'
                t.sides[N].strength = 1
                t.sides[E].label = f'{lcol}>{col}{col_mark}{diagonal_signal}'
                t.sides[E].strength = 1
                t.sides[S].label = f'{col}{col_mark}{diagonal_signal}'
                t.sides[S].strength = 1
                t.sides[W].label = f'{lcol}>'
                t.sides[W].strength = 1
                if b_3D:
                    t.sides[U].label = f'{col}{col_mark}'
                    t.sides[U].strength = 1
                t.color = color_mapper(1, col, 'B', col_mark)
                tile_set[tile_type_name] = t
                
                
                for col2 in [0,1]:
                    for col_mark2 in ['B', 'I']:

                        # Make the tile types that get the star and one tile's info from the west and pass it east
                        tile_type_name = f'LtoR1_{lcol}_{col}{col_mark}{col2}{col_mark2}{diagonal_signal}'
                        tile_type_label = tile_type_name
                        t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                        t.sides[N].label = f'{col2}{col_mark2}'
                        t.sides[N].strength = 1
                        t.sides[W].label = f'{lcol}>{col}{col_mark}{diagonal_signal}'
                        t.sides[W].strength = 1
                        t.sides[E].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}{diagonal_signal}'
                        t.sides[E].strength = 1
                        t.sides[S].label = f'{col2}{col_mark2}'
                        t.sides[S].strength = 1
                        if b_3D:
                            t.sides[U].label = f'{col2}{col_mark2}'
                            t.sides[U].strength = 1
                        t.color = color_mapper(1, col2, 'B', col_mark2)
                        tile_set[tile_type_name] = t

                        for col3 in [0,1]:
                            for col_mark3 in ['B', 'I']:

                                # Make the tile types that get the star and two tiles' info from the west and pass three east
                                tile_type_name = f'LtoR3_{lcol}_{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}{diagonal_signal}'
                                tile_type_label = tile_type_name
                                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                                t.sides[N].label = f'{col3}{col_mark3}'
                                t.sides[N].strength = 1
                                t.sides[W].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}{diagonal_signal}'
                                t.sides[W].strength = 1
                                t.sides[E].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}{diagonal_signal}'
                                t.sides[E].strength = 1
                                t.sides[S].label = f'{col3}{col_mark3}'
                                t.sides[S].strength = 1
                                if b_3D:
                                    t.sides[U].label = f'{col3}{col_mark3}'
                                    t.sides[U].strength = 1
                                t.color = color_mapper(1, col3, 'B', col_mark3)
                                tile_set[tile_type_name] = t


                                for col4 in [0,1]:
                                    for col_mark4 in ['B', 'I']:

                                        # Make the tile types that get three tiles' info from the west and pass it east
                                        tile_type_name = f'LtoR14_{lcol}_{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}{col4}{col_mark4}{diagonal_signal}'
                                        tile_type_label = tile_type_name
                                        t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                                        t.sides[N].label = f'{col4}{col_mark4}{diagonal_signal}'
                                        t.sides[N].strength = 1
                                        t.sides[W].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}{diagonal_signal}'
                                        t.sides[W].strength = 1
                                        t.sides[E].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}'
                                        t.sides[E].strength = 1
                                        t.sides[S].label = f'{col4}{col_mark4}'
                                        t.sides[S].strength = 1
                                        if b_3D:
                                            t.sides[U].label = f'{col4}{col_mark4}'
                                            t.sides[U].strength = 1
                                        t.color = color_mapper(1, col4, 'B', col_mark4)
                                        tile_set[tile_type_name] = t

                                        # Make the tile types that get three tiles' info from the west and pass it east
                                        tile_type_name = f'LtoR14_{lcol}_{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}{col4}{col_mark4}'
                                        tile_type_label = tile_type_name
                                        t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                                        t.sides[N].label = f'{col4}{col_mark4}'
                                        t.sides[N].strength = 1
                                        t.sides[W].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}'
                                        t.sides[W].strength = 1
                                        t.sides[E].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}'
                                        t.sides[E].strength = 1
                                        t.sides[S].label = f'{col4}{col_mark4}'
                                        t.sides[S].strength = 1
                                        if b_3D:
                                            t.sides[U].label = f'{col4}{col_mark4}'
                                            t.sides[U].strength = 1
                                        t.color = color_mapper(1, col4, 'B', col_mark4)
                                        tile_set[tile_type_name] = t
                                                

                                # Make the tiles that have a blank to the south and receive three tiles' info from the west
                                tile_type_name = f'LtoR15_{lcol}_{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}'
                                tile_type_label = tile_type_name
                                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                                t.sides[N].label = f'{col}{col_mark}'
                                t.sides[N].strength = 1
                                t.sides[W].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}{col3}{col_mark3}'
                                t.sides[W].strength = 1
                                t.sides[E].label = f'{lcol}>>{col2}{col_mark2}{col3}{col_mark3}'
                                t.sides[E].strength = 1
                                t.sides[S].label = f'__'
                                t.sides[S].strength = 1
                                if b_3D:
                                    t.sides[U].label = f'{col}{col_mark}'
                                    t.sides[U].strength = 1
                                t.color = color_mapper(1, col, 'B', col_mark)
                                tile_set[tile_type_name] = t

                        # Make the tiles that have a blank to the south and receive two tiles' info from the west
                        tile_type_name = f'LtoR16_{lcol}_{col}{col_mark}{col2}{col_mark2}'
                        tile_type_label = tile_type_name
                        t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                        t.sides[N].label = f'{col}{col_mark}'
                        t.sides[N].strength = 1
                        t.sides[W].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}'
                        t.sides[W].strength = 1
                        t.sides[E].label = f'{lcol}>>{col2}{col_mark2}'
                        t.sides[E].strength = 1
                        t.sides[S].label = f'__'
                        t.sides[S].strength = 1
                        if b_3D:
                            t.sides[U].label = f'{col}{col_mark}'
                            t.sides[U].strength = 1
                        t.color = color_mapper(1, col, 'B', col_mark)
                        tile_set[tile_type_name] = t

                # Make the tiles that have a blank to the south and receive one tile's info from the west
                tile_type_name = f'LtoR116_{lcol}_{col}{col_mark}'
                tile_type_label = tile_type_name
                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                t.sides[N].label = f'{col}{col_mark}'
                t.sides[N].strength = 1
                t.sides[W].label = f'{lcol}>>{col}{col_mark}'
                t.sides[W].strength = 1
                t.sides[E].label = f'>>'
                t.sides[E].strength = 1
                t.sides[S].label = f'__'
                t.sides[S].strength = 1
                if b_3D:
                    t.sides[U].label = f'{col}{col_mark}'
                    t.sides[U].strength = 1
                t.color = color_mapper(1, col, 'B', col_mark)
                tile_set[tile_type_name] = t

    # Make the tile that has a blank to the south and receive no tile's info from the west
    tile_type_name = f'LtoR7'
    tile_type_label = tile_type_name
    t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
    t.sides[N].label = f'__'
    t.sides[N].strength = 1
    t.sides[W].label = f'>>'
    t.sides[W].strength = 1
    t.sides[E].label = f'>>'
    t.sides[E].strength = 1
    t.sides[S].label = f'__'
    t.sides[S].strength = 1
    t.color = color_yellow
    tile_set[tile_type_name] = t

    tile_type_name = f'LtoR8'
    tile_type_label = tile_type_name
    t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
    t.sides[N].label = f'__'
    t.sides[N].strength = 1
    t.sides[W].label = f'>>'
    t.sides[W].strength = 1
    t.sides[E].label = f'>>R'
    t.sides[E].strength = 2
    t.sides[S].label = f'__R'
    t.sides[S].strength = 1
    t.color = color_yellow
    tile_set[tile_type_name] = t

    tile_type_name = f'LtoR9'
    tile_type_label = tile_type_name
    t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
    t.sides[N].label = f'__R'
    t.sides[N].strength = 2
    t.sides[W].label = f'>>R'
    t.sides[W].strength = 2
    t.sides[S].label = f'fill'
    t.sides[S].strength = 1
    t.sides[E].label = f'fill'
    t.sides[E].strength = 1
    t.color = color_yellow
    tile_set[tile_type_name] = t


    ##########################################################################################
    #### Right-to-left tiles
    tile_type_name = f'RtoL1'
    tile_type_label = tile_type_name
    t = PyTASTileType(name=tile_type_name, label=tile_type_label)
    t.sides[N].label = f'__'
    t.sides[N].strength = 1
    t.sides[W].label = f'<'
    t.sides[W].strength = 1
    t.sides[E].label = f'R<'
    t.sides[E].strength = 2
    t.sides[S].label = f'__R'
    t.sides[S].strength = 2
    t.color = color_yellow
    tile_set[tile_type_name] = t

    tile_type_name = f'RtoL2'
    tile_type_label = tile_type_name
    t = PyTASTileType(name=tile_type_name, label=tile_type_label)
    t.sides[N].label = f'__R'
    t.sides[N].strength = 1
    t.sides[W].label = f'R<'
    t.sides[W].strength = 2
    t.sides[S].label = f'fill'
    t.sides[S].strength = 1
    t.sides[E].label = f'fill'
    t.sides[E].strength = 1
    t.color = color_yellow
    tile_set[tile_type_name] = t

    tile_type_name = f'RtoL3'
    tile_type_label = tile_type_name
    t = PyTASTileType(name=tile_type_name, label=tile_type_label)
    t.sides[N].label = f'__'
    t.sides[N].strength = 1
    t.sides[W].label = f'<'
    t.sides[W].strength = 1
    t.sides[E].label = f'<'
    t.sides[E].strength = 1
    t.sides[S].label = f'__'
    t.sides[S].strength = 1
    t.color = color_yellow
    tile_set[tile_type_name] = t

    for col in [0,1]:
        for col_mark in ['B', 'I']:
            tile_type_name = f'RtoL4{col}{col_mark}'
            tile_type_label = tile_type_name
            t = PyTASTileType(name=tile_type_name, label=tile_type_label)
            t.sides[N].label = f'{col}{col_mark}'
            t.sides[N].strength = 1
            t.sides[W].label = f'<'
            t.sides[W].strength = 1
            t.sides[E].label = f'<'
            t.sides[E].strength = 1
            t.sides[S].label = f'{col}{col_mark}'
            t.sides[S].strength = 1
            t.color = color_mapper(1, col, 'B', col_mark)
            tile_set[tile_type_name] = t

            tile_type_name = f'RtoL5{col}{col_mark}{diagonal_signal}'
            tile_type_label = tile_type_name
            t = PyTASTileType(name=tile_type_name, label=tile_type_label)
            t.sides[N].label = f'{col}{col_mark}{diagonal_signal}'
            t.sides[N].strength = 1
            t.sides[W].label = f'<'
            t.sides[W].strength = 1
            t.sides[E].label = f'<'
            t.sides[E].strength = 1
            t.sides[S].label = f'{col}{col_mark}{diagonal_signal}'
            t.sides[S].strength = 1
            t.color = color_mapper(1, col, 'B', col_mark)
            tile_set[tile_type_name] = t

            tile_type_name = f'RtoL4{col}{col_mark}L'
            tile_type_label = tile_type_name
            t = PyTASTileType(name=tile_type_name, label=tile_type_label)
            t.sides[N].label = f'L{col}{col_mark}'
            t.sides[N].strength = 2
            t.sides[W].label = f'<'
            t.sides[W].strength = 1
            t.sides[E].label = f'<'
            t.sides[E].strength = 1
            t.sides[S].label = f'L{col}{col_mark}'
            t.sides[S].strength = 1
            t.color = color_mapper(1, col, 'B', col_mark)
            tile_set[tile_type_name] = t


    # Make the tiles for the right side that receive info for one, two, or three tiles from the left
    for col in [0,1]:
        for col_mark in ['B', 'I']:

            for lcol in [0, 1]:
                # Make the tiles that have a blank to the south and receive one tile's info from the west
                tile_type_name = f'LtoR26_{lcol}_{col}{col_mark}R'
                tile_type_label = tile_type_name
                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                t.sides[N].label = f'{col}{col_mark}'
                t.sides[N].strength = 1
                t.sides[W].label = f'{lcol}>>{col}{col_mark}'
                t.sides[W].strength = 1
                t.sides[E].label = f'>>R'
                t.sides[E].strength = 2
                t.sides[S].label = f'__R'
                t.sides[S].strength = 1
                if b_3D:
                    t.sides[U].label = f'{col}{col_mark}'
                    t.sides[U].strength = 1
                t.color = color_mapper(1, col, 'B', col_mark)
                tile_set[tile_type_name] = t

                for col2 in [0,1]:
                    for col_mark2 in ['B', 'I']:

                        # Make the tiles that have a blank to the south and receive two tiles' info from the west
                        tile_type_name = f'LtoR26_{lcol}_{col}{col_mark}{col2}{col_mark2}R'
                        tile_type_label = tile_type_name
                        t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                        t.sides[N].label = f'{col}{col_mark}'
                        t.sides[N].strength = 1
                        t.sides[W].label = f'{lcol}>>{col}{col_mark}{col2}{col_mark2}'
                        t.sides[W].strength = 1
                        t.sides[E].label = f'{lcol}>>{col2}{col_mark2}R'
                        t.sides[E].strength = 2
                        t.sides[S].label = f'__R'
                        t.sides[S].strength = 1
                        if b_3D:
                            t.sides[U].label = f'{col}{col_mark}'
                            t.sides[U].strength = 1
                        t.color = color_mapper(1, col, 'B', col_mark)
                        tile_set[tile_type_name] = t

                # Make the tiles that receive one tile's info from the west and extend to the east to complete the assembly
                tile_type_name = f'LtoR27_{lcol}_{col}{col_mark}R'
                tile_type_label = tile_type_name
                t = PyTASTileType(name=tile_type_name, label=tile_type_label, is3D=b_3D)
                t.sides[N].label = f'{col}{col_mark}R'
                t.sides[N].strength = 1
                t.sides[W].label = f'{lcol}>>{col}{col_mark}R'
                t.sides[W].strength = 2
                t.sides[S].label = f'fill'
                t.sides[S].strength = 1
                if b_3D:
                    t.sides[U].label = f'{lcol}_{col}{col_mark}R'
                    t.sides[U].strength = 2
                t.color = color_red
                tile_set[tile_type_name] = t

    if b_filler:
        tile_type_name = f'Filler'
        tile_type_label = f'Fill'
        t = PyTASTileType(name=tile_type_name, label=tile_type_label)
        t.sides[N].label = f'fill'
        t.sides[N].strength = 1
        t.sides[E].label = f'fill'
        t.sides[E].strength = 1
        t.sides[S].label = f'fill'
        t.sides[S].strength = 1
        t.sides[W].label = f'fill'
        t.sides[W].strength = 1
        t.color = color_grey
        tile_set[tile_type_name] = t
            
    return tile_set

    
if __name__ == "__main__":
    try:
        # starts at the second element of argv since the first one is the script name
        # extraparams are extra arguments passed after all option/keywords are assigned
        # opts is a list containing the pair "option"/"value"
        opts, extraparams = getopt.getopt(sys.argv[1:], "hp:w:o:d:ft",
                            ["help", "pattern=", "width=", "out=", "dir=", "fill", "three"])
        #print 'Opts:',opts
        #print 'Extra parameters:',extraparams
    except:
        print("")
        print("Unrecognized option:" + str(sys.argv[1:]))
        DisplayHelp()


    # Default values (used if not specified on the command line)
    pattern = '01101011'
    width = None

    base_file_name = None
    output_directory = '.'

    b_mark_signals = False
    b_filler = False
    b_3D = False

    for o,p in opts:
        if o in ['-h', '--help']:
            DisplayHelp()
        elif o in ['-p', '--pattern']:
            pattern = p
        elif o in ['-w', '--width']:
            width = int(p)
        elif o in ['-o', '--out']:
            base_file_name = p
        elif o in ['-d', '--dir']:
            output_directory = p
        elif o in ['-f', '--fill']:
            b_filler = True
        elif o in ['-t', '--three']:
            b_3D = True
        else:
            DisplayHelp()

    if width is None:
        width = len(pattern) + 10
    elif width < len(pattern):
        print(f'\nERROR: The width of the starting row must be at least that of the pattern.\n')
        DisplayHelp()

    if base_file_name is None:
        base_file_name = f'pattern_{pattern}_width{width}'
        if b_3D:
            base_file_name += '_3D'
    
    if not os.path.exists(output_directory):
        print(f'Creating output directory {output_directory}')
        os.makedirs(output_directory)
   
    expanded_pattern = ''
    for j in range(len(pattern)):
        if j == 0:
            expanded_pattern += pattern[j] + 'B'
        else:
            expanded_pattern += pattern[j] + 'I'
    for i in range(width-len(pattern)):
        expanded_pattern += '__'

    print(f'expanded_pattern: {expanded_pattern}')

    diagonal_signal = 'X'
    top_diagonal_signal = '$'
    signal_symbols = [diagonal_signal, top_diagonal_signal]

    tas = PyTASTileSystem()

    init_row_tile_set, init_row_tile_list = GetExtenderInitialRowTiles(expanded_pattern, ColorMapper, 2, diagonal_signal)
    tas.tileTypes.update(init_row_tile_set)

    extender_tiles = GetExtenderTiles(ColorMapper, diagonal_signal, top_diagonal_signal, b_3D)
    tas.tileTypes.update(extender_tiles)

    if b_3D:
        second_plane_tiles = GetSecondPlaneFirstRowTiles(ColorMapper, top_diagonal_signal)
        tas.tileTypes.update(second_plane_tiles)
        
        grid_tiles = GetInteriorTiles(ColorMapper, top_diagonal_signal)
        tas.tileTypes.update(grid_tiles)

##    if b_mark_signals:
##        for tile_type_name in tas.tileTypes:
##            tile_type = tas.tileTypes[tile_type_name]
##            for symbol in signal_symbols:
##                if tile_type.label.endswith(symbol):
##                    tile_type.color = color_red

    x_coord = 0
    for t in init_row_tile_list:
        tas.seedPatch.append((t.name, (x_coord, 0)))
        x_coord += 1

    tas.SaveToFile(f'{output_directory}/{base_file_name}')
    text_3D = ''
    if b_3D:
        text_3D = f' plus {len(second_plane_tiles)} for the first row of z=1, plus {len(grid_tiles)} to create the grid.'
    
    out_str = f"Output TAS to files '{output_directory}/{base_file_name}' with {len(tas.tileTypes)} tile types for pattern '{pattern}' and width {width}."
    out_str += f' Created {len(init_row_tile_set)} tile types for the seed row, plus {len(extender_tiles)} for the extender tiles'
    out_str += text_3D
    print(out_str)

    


