import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Scanner;

//This program takes as input a pattern, number of times to repeat it, 
//and optionally the name of any output files. Output files contain an aTAM system which 
//creates a grid of that pattern which is input, which may expand infinitely from the seed tile. 
//The user may mark bounding signal tiles in black, and growth signal tiles in red. 


//Made by Phillip Drake, University of Arkansas, 02.11.2024
//If you have any questions or concerns, email me at padrake@uark.edu
public class GridGenerator {
    public static void main(String[] args) throws IOException{
        boolean markSignals = false, markBounds = false, tileInfinite = false;
        boolean inputPattern = false, inputRepetition = false, userOutput = false;
        boolean menu = false;
        boolean nextIsPattern = false, nextIsRepetition = false, nextIsOutputFile = false;
        String fileName = "";
        //makes output directory if it does not exist already
        File outputDirectory = new File("output/");
        if(!outputDirectory.exists()){
            outputDirectory.mkdir();
        }
        String pattern = "10101010";
        int numRepetitions = 10;
        ArrayList<Tile> tiles = new ArrayList<Tile>();
        //command line argument parsing
        for (String s: args){
            if(s.equals("-menu"))
            {
                menu = true;
            }
            else if(menu){
                break;
            }
            else if(s.equals("-p"))
            {
                inputPattern = true;
                nextIsPattern = true;
            }
            else if(nextIsPattern){
                pattern = s;
                nextIsPattern = false;
            }
            else if(s.equals("-r"))
            {
                inputRepetition = true;
                nextIsRepetition = true;
            }
            else if(nextIsRepetition){
                numRepetitions = Integer.parseInt(s);
                nextIsRepetition = false;
            } else if(s.equals("-o")){
                userOutput = true;
                nextIsOutputFile = true;
            } else if(nextIsOutputFile){
                fileName = s;
                nextIsOutputFile = false;
            }
            else if(s.equals("-s"))
            {
                markSignals = true;
            }
            else if(s.equals("-b"))
            {
                markBounds = true;
            }
            else if(s.equals("-i"))
            {
                tileInfinite = true;
            }
        }
        


        Scanner s = new Scanner(System.in); 
        if(menu){
            System.out.println("Input pattern");
            pattern = s.nextLine();
        }
        int initialLength = pattern.length();

        if(menu){
            System.out.println("Input number of repetitions");
            numRepetitions = Integer.parseInt(s.nextLine());
        }
        //repeats pattern numRepetition times
        String finalPattern = pattern;
        for(int i = 1; i < numRepetitions; i++){
            finalPattern += pattern;
        }

        if(menu){
            System.out.println("Input output file name");
            fileName = s.nextLine();
        }
        else if(!userOutput){
            fileName+="pattern_" + pattern + "_x" + numRepetitions;
        }
        if(menu){
            System.out.println("Would you like to tile infinitely? (y/n)");
            if(s.nextLine().toLowerCase().equals("y"))
                tileInfinite = true;
        }

        if(tileInfinite){
            getInitialRow(tiles, finalPattern, initialLength, true);
            getNorthTiles(tiles, finalPattern);
            getSouthTiles(tiles, finalPattern, initialLength);
            getEastTile(tiles, finalPattern, initialLength);
            getWestTiles(tiles, finalPattern);
        } else {
            getInitialRow(tiles, finalPattern, initialLength, false);
            getNonInfiniteTiles(tiles, finalPattern);
        }
        colorTiles(tiles, pattern.charAt(0)=='0');
        
        if(menu)
        {
            System.out.println("Would you like to mark creation signals? (y/n)");            
            if(s.nextLine().toLowerCase().equals("y")){  
                markSignals = true;
            }
        }
        if(markSignals)
            markSignals(tiles);

        if(menu){
            System.out.println("Would you like to mark bounds? (y/n)");
            if(s.nextLine().toLowerCase().equals("y")){
                markBounds = true;
            }
        }
        if(markBounds)
            markBounds(tiles);

        //writes tiles to output file
        FileWriter fw = new FileWriter(new File(outputDirectory + "/" + fileName+".tds"));
        for(int i = 0; i < tiles.size(); i++){
            fw.write(tiles.get(i).toString());
        }
        fw.close();
        generateTDPFile(fileName, outputDirectory.toString(), finalPattern);
    }
    //generates a .tdp file
    public static void generateTDPFile(String fileName, String outputDirectory, String pattern) throws IOException{
        FileWriter fw = new FileWriter(outputDirectory + "/" + fileName+".tdp");
        fw.write(fileName+".tds\n");
        fw.write("Temperature=2\n");
        for(int i = 0; i < pattern.length();i++){
            fw.write("Init"+i*2+" "+i+" 0 0\n");
        }
        fw.close();
    }
    public static void getInitialRow(ArrayList<Tile> tiles, String pattern, int initialLength, boolean bounds){
        Tile t = new Tile();
        if(bounds) {
            t.setName("Init0");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(2);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(0)+"BLBN");
            t.setEastLabel("init" + 0);
            t.setWestLabel(pattern.charAt(0) + "BTBW");
            t.setSouthLabel(pattern.charAt(0)+"BLBI");
            tiles.add(t);
            t = new Tile();
        }
        for(int i = (bounds) ? 1 : 0; i < pattern.length()-1; i++){
            t.setName("Init" + i*2);
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(2);
            t.setSouthStrength(1);
            t.setWestStrength(2);
            t.setNorthLabel(pattern.charAt(i) + "I");
            t.setEastLabel("init" + i*2);
            t.setWestLabel("init" + (i-1)*2);
            t.setSouthLabel(pattern.charAt(i)+"I");
            if(i==1){
                t.setSouthLabel(pattern.charAt(i)+"I*");
                t.setSouthStrength(2);
            }
            if(i % initialLength == 0){
                t.setNorthLabel(pattern.charAt(i)+"B");
                t.setSouthLabel(pattern.charAt(i)+"B");
            }
            tiles.add(t);
            t = new Tile();
        }
        if(!bounds){
            t.setName("Init"+(pattern.length()-1)*2);
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(2);
            t.setSouthStrength(1);
            t.setWestStrength(2);
            t.setNorthLabel("");
            t.setEastLabel("");
            t.setSouthLabel(pattern.charAt(pattern.length()-1)+"I");
            t.setWestLabel("init" + (pattern.length()-2)*2);
            tiles.add(t);
            t = new Tile();
        } else if(pattern.length() > 2){
            t.setName("Init"+(pattern.length()-1)*2);
            t.setLabel(t.name);
            t.setNorthStrength(2);
            t.setEastStrength(2);
            t.setSouthStrength(1);
            t.setWestStrength(2);
            t.setNorthLabel(pattern.charAt(pattern.length()-1)+"I*NI");
            t.setEastLabel(pattern.charAt(0) + "B*EI");
            t.setSouthLabel(pattern.charAt(pattern.length()-1)+"IRBI");
            t.setWestLabel("init" + (pattern.length()-2)*2);
            tiles.add(t);
            t = new Tile();
        } else {
            t.setName("Init"+(pattern.length()-1)*2);
            t.setLabel(t.name);
            t.setNorthStrength(2);
            t.setEastStrength(2);
            t.setSouthStrength(2);
            t.setWestStrength(2);
            t.setNorthLabel(pattern.charAt(pattern.length()-1)+"I*NI");
            t.setEastLabel(pattern.charAt(0) + "B*EI");
            t.setSouthLabel(pattern.charAt(pattern.length()-1)+"IRB*I");
            t.setWestLabel("init" + (pattern.length()-2)*2);
            tiles.add(t);
            t = new Tile();
        }
    }
    public static void getNonInfiniteTiles(ArrayList<Tile> tiles, String pattern){
        Tile t = new Tile();
        String[] modes = {"0I", "1I", pattern.charAt(0)+"B"};
        //< and > tiles
        for(String c : modes){
            for(String d : modes){
                t.setName("<"+c+d);
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(d);
                t.setEastLabel("<"+c);
                t.setSouthLabel(d);
                t.setWestLabel("<"+c);
                tiles.add(t);
                t = new Tile();
                t.setName(c+d+">");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(d);
                t.setEastLabel(c+">");
                t.setSouthLabel(d);
                t.setWestLabel(c+">");
                tiles.add(t);
                t = new Tile();
            }
        }
        // * tiles
        for(String c : modes){
            t.setName(c+c+"*");
            t.setLabel(t.name);
            t.setNorthStrength(2);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(c+"*");
            t.setEastLabel(c+"*");
            t.setSouthLabel(c);
            t.setWestLabel("<"+c);
            tiles.add(t);
            t = new Tile();
        }
        //>* tiles
        String[] types = {"0", "1"};
        String[] types2 = {"B", "I"};
        for(String col : types){
            for(String colMark : types2) {
                for(String row : types){
                    for(String rowMark : types2){
                        t.setName(row + rowMark + col + colMark + ">*");
                        t.setLabel(t.name);
                        t.setNorthStrength(1);
                        t.setEastStrength(1);
                        t.setSouthStrength(2);
                        t.setWestStrength(1);
                        t.setNorthLabel(col+colMark);
                        t.setEastLabel(row+rowMark+">");
                        t.setSouthLabel(col+colMark+"*");
                        t.setWestLabel(row+rowMark+"*");
                        tiles.add(t);
                        t = new Tile();
                    }
                }
            }
        }
    }   
     //create filler tiles for the first segment of spiralization
    public static void getSouthTiles(ArrayList<Tile> tiles, String pattern, int initialLength){
        Tile t = new Tile();
        String[] modes = {"0I","1I",pattern.charAt(0)+"B"};
        //< and > tiles
        for(String c : modes){
            for(String d : modes){
                t.setName("<"+c+d);
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(d);
                t.setEastLabel("<"+c);
                t.setSouthLabel(d);
                t.setWestLabel("<"+c);
                tiles.add(t);
                t = new Tile();
                t.setName(c+d+">");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(d);
                t.setEastLabel(c+">");
                t.setSouthLabel(d);
                t.setWestLabel(c+">");
                tiles.add(t);
                t = new Tile();
            }
        }
        

        // * tiles
        for(String c : modes){
            t.setName(c+c+"*");
            t.setLabel(t.name);
            t.setNorthStrength(2);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(c+"*");
            t.setEastLabel(c+"*");
            t.setSouthLabel(c);
            t.setWestLabel("<"+c);
            tiles.add(t);
            t = new Tile();
        }
        //>* tiles
        String[] types = {"0", "1"};
        String[] types2 = {"B", "I"};
        for(String col : types){
            for(String colMark : types2) {
                for(String row : types){
                    for(String rowMark : types2){
                        t.setName(row + rowMark + col + colMark + ">*");
                        t.setLabel(t.name);
                        t.setNorthStrength(1);
                        t.setEastStrength(1);
                        t.setSouthStrength(2);
                        t.setWestStrength(1);
                        t.setNorthLabel(col+colMark);
                        t.setEastLabel(row+rowMark+">");
                        t.setSouthLabel(col+colMark+"*");
                        t.setWestLabel(row+rowMark+"*");
                        tiles.add(t);
                        t = new Tile();
                    }
                }
            }
        }

        //Bounding tiles!!!
        for(String c : modes){
            //First left bound (must exist in order to build west deterministically)
            t.setName("<" + c + pattern.charAt(0) + "BLBI");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(0)+"BLBI");
            t.setEastLabel("<" + c);
            t.setSouthLabel(pattern.charAt(0)+"BLBI");
            t.setWestLabel(c+">W");
            tiles.add(t);
            t = new Tile();
            //Subsequent left bounds
            t.setName("<" + c + pattern.charAt(0) + "BLB");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(0)+"BLB");
            t.setEastLabel("<" + c);
            t.setSouthLabel(pattern.charAt(0)+"BLB");
            t.setWestLabel("<" + c);
            tiles.add(t);
            t = new Tile();
            //first right bound tiles
            t.setName(c + pattern.charAt(pattern.length() - 1) + "IRB>I");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(pattern.length()-1)+"IRBI");
            t.setEastLabel("<"+c);
            t.setSouthLabel(pattern.charAt(pattern.length()-1)+"IRBI");
            t.setWestLabel(c+">");
            tiles.add(t);
            t = new Tile();
            //subsequent right bound tiles
            t.setName(c + pattern.charAt(pattern.length() - 1) + "IRB>");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(pattern.length()-1)+"IRB");
            t.setEastLabel("<"+c);
            t.setSouthLabel(pattern.charAt(pattern.length()-1)+"IRB");
            t.setWestLabel(c+">");
            tiles.add(t);
            t = new Tile();
            //bottom bound tiles
            t.setName("<" + pattern.charAt(pattern.length() - 1) + "I" + c + "BB");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(c);
            t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBB");
            t.setSouthLabel(c);
            t.setWestLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBB");
            tiles.add(t);
            t = new Tile();
        }
        //Connector Tiles
        if(pattern.length() > 2){
            if(initialLength != 2){
                t.setName(pattern.charAt(pattern.length() - 2) + "I" + pattern.charAt(pattern.length() - 1) + "IRB>*");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(2);
                t.setWestStrength(1);
                t.setNorthLabel(pattern.charAt(pattern.length() - 1) + "IRB");
                t.setEastLabel("<" + pattern.charAt(pattern.length() - 2) + "I");
                t.setSouthLabel(pattern.charAt(pattern.length() - 1) + "IRB*");
                t.setWestLabel(pattern.charAt(pattern.length() - 2) + "I*");
                tiles.add(t);
                t = new Tile();
                t.setName(pattern.charAt(0) + "B" + pattern.charAt(pattern.length() - 1) + "IRB>*I");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(2);
                t.setWestStrength(1);
                t.setNorthLabel(pattern.charAt(pattern.length() - 1) + "IRBI");
                t.setEastLabel("<" + pattern.charAt(pattern.length() - 2) + "I");
                t.setSouthLabel(pattern.charAt(pattern.length() - 1) + "IRB*I");
                t.setWestLabel(pattern.charAt(pattern.length() - 2) + "I*");
                tiles.add(t);
                t = new Tile();
            } else {
                t.setName(pattern.charAt(pattern.length() - 2) + "I" + pattern.charAt(pattern.length() - 1) + "IRB>*");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(2);
                t.setWestStrength(1);
                t.setNorthLabel(pattern.charAt(pattern.length() - 1) + "IRB");
                t.setEastLabel("<" + pattern.charAt(pattern.length() - 2) + "B");
                t.setSouthLabel(pattern.charAt(pattern.length() - 1) + "IRB*");
                t.setWestLabel(pattern.charAt(pattern.length() - 2) + "B*");
                tiles.add(t);
                t = new Tile();
                t.setName(pattern.charAt(0) + "B" + pattern.charAt(pattern.length() - 1) + "IRB>*I");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(2);
                t.setWestStrength(1);
                t.setNorthLabel(pattern.charAt(pattern.length() - 1) + "IRBI");
                t.setEastLabel("<" + pattern.charAt(pattern.length() - 2) + "B");
                t.setSouthLabel(pattern.charAt(pattern.length() - 1) + "IRB*I");
                t.setWestLabel(pattern.charAt(pattern.length() - 2) + "B*");
                tiles.add(t);
                t = new Tile();
            }
        } else {
            t.setName(pattern.charAt(0) + "B" + pattern.charAt(pattern.length() - 1) + "IRB>*");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(2);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(pattern.length() - 1) + "IRB");
            t.setEastLabel("<" + pattern.charAt(pattern.length() - 2) + "B");
            t.setSouthLabel(pattern.charAt(pattern.length() - 1) + "IRB*");
            t.setWestLabel(pattern.charAt(pattern.length() - 2) + "B*");
            tiles.add(t);
            t = new Tile();
            t.setName(pattern.charAt(0) + "B" + pattern.charAt(pattern.length() - 1) + "IRB>*I");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(2);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(pattern.length() - 1) + "IRBI");
            t.setEastLabel("<" + pattern.charAt(pattern.length() - 2) + "B");
            t.setSouthLabel(pattern.charAt(pattern.length() - 1) + "IRB*I");
            t.setWestLabel(pattern.charAt(pattern.length() - 2) + "B*");
            tiles.add(t);
            t = new Tile();
        }

        t.setName(pattern.charAt(pattern.length() - 1) + "I" + pattern.charAt(pattern.length() - 1) + "IRB*I");
        t.setLabel(t.name);
        t.setNorthStrength(2);
        t.setEastStrength(1);
        t.setSouthStrength(1);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(pattern.length() - 1) + "IRB*I");    
        t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE");
        t.setSouthLabel(pattern.charAt(pattern.length() - 1) + "IRB");
        t.setWestLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBB");
        tiles.add(t);
        t = new Tile();
        t.setName(pattern.charAt(pattern.length() - 1) + "I" + pattern.charAt(pattern.length() - 1) + "IRB*");
        t.setLabel(t.name);
        t.setNorthStrength(2);
        t.setEastStrength(1);
        t.setSouthStrength(1);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(pattern.length() - 1) + "IRB*");
        t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "I");
        t.setSouthLabel(pattern.charAt(pattern.length() - 1) + "IRB");
        t.setWestLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBB");
        tiles.add(t);
        t = new Tile();
        t.setName("<"+pattern.charAt(pattern.length()-1)+"I"+pattern.charAt(0)+"B"+"LBIE");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(1);
        t.setSouthStrength(2);
        t.setWestStrength(2);
        t.setNorthLabel(pattern.charAt(0)+"BLBI");
        t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBB");
        t.setSouthLabel(pattern.charAt(0)+"B*LB");
        t.setWestLabel(pattern.charAt((pattern.length()-1))+"I*BBW");
        tiles.add(t);
        t = new Tile();
        t.setName(pattern.charAt(0)+"B"+pattern.charAt(0)+"B*LB");
        t.setLabel(t.name);
        t.setNorthStrength(2);
        t.setEastStrength(1);
        t.setSouthStrength(1);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(0)+"B*LB");
        t.setEastLabel(pattern.charAt(0)+"B*");
        t.setSouthLabel(pattern.charAt(0)+"BLB");
        t.setWestLabel("<" + pattern.charAt(0)+"B");
        tiles.add(t);
        t = new Tile();
        t.setName("<"+pattern.charAt(pattern.length()-1)+"I"+pattern.charAt(0)+"BBL");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(1);
        t.setSouthStrength(2);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(0)+"BLB");
        t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBB");
        t.setSouthLabel(pattern.charAt(0)+"B*LB");
        t.setWestLabel("<" + pattern.charAt((pattern.length()-1))+"I");
        tiles.add(t);
        t = new Tile();


    }
    public static void getEastTile(ArrayList<Tile> tiles, String pattern, int initialLength){
        Tile t = new Tile();
        String sideLabel = "", topLabel = "";
        String[] modes = {"0I","1I",pattern.charAt(0)+"B"};
        //east-tile generation
        t.setName(pattern.charAt(0)+"B"+pattern.charAt(0)+"B*TB");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(1);
        t.setSouthStrength(1);
        t.setWestStrength(2);
        t.setNorthLabel(pattern.charAt(0)+"B");
        t.setEastLabel(pattern.charAt(0)+"BTBE");
        t.setSouthLabel(pattern.charAt(0)+"B*E");
        t.setWestLabel(pattern.charAt(0)+"B*EI");
        tiles.add(t);
        t = new Tile();
        //* tiles
        for(String c : modes){
            t.setName(c+c+"*E");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(2);
            t.setNorthLabel(c);
            t.setEastLabel(c+">");
            t.setSouthLabel(c+"*E");
            t.setWestLabel(c+"*E");
            tiles.add(t);
            t = new Tile();
        }
        //<* tiles
        String[] types = {"0", "1"};
        String[] types2 = {"B", "I"};
        for(String col : types){
            for(String colMark : types2) {
                for(String row : types){
                    for(String rowMark : types2){
                        t.setName("<" +row + rowMark + col + colMark + "*E");
                        t.setLabel(t.name);
                        t.setNorthStrength(1);
                        t.setEastStrength(2);
                        t.setSouthStrength(1);
                        t.setWestStrength(1);
                        t.setNorthLabel(col+colMark+"*E");
                        t.setEastLabel(row+rowMark+"*E");
                        t.setSouthLabel(col+colMark);
                        t.setWestLabel("<" +row+rowMark);
                        tiles.add(t);
                        t = new Tile();
                    }
                }
            }
        }
        //Bounding tiles
        for(String c : modes) {
            //bottom bound
            t.setName("<" + pattern.charAt(pattern.length() - 1) + "I" + c + "BBE");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(c);
            t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE");
            t.setSouthLabel(c);
            t.setWestLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE");
            tiles.add(t);
            t = new Tile();
            //right bound
            t.setName(c + pattern.charAt(pattern.length() - 1) + "IRB>E");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(pattern.length()-1)+"IRBE");
            t.setEastLabel("<"+c);
            t.setSouthLabel(pattern.charAt(pattern.length()-1)+"IRBE");
            t.setWestLabel(c+">");
            tiles.add(t);
            t = new Tile();
            //top bound
            t.setName(pattern.charAt(0)+"B"+c+"TBE");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(c);
            t.setEastLabel(pattern.charAt(0)+"BTBE");
            t.setSouthLabel(c);
            t.setWestLabel(pattern.charAt(0)+"BTBE");
            tiles.add(t);
            t = new Tile();

        }
        //End Tiles
        if(pattern.length() > 2){
            if(initialLength != 2){
                t.setName("<" + pattern.charAt(pattern.length()-1)+"I"+pattern.charAt(pattern.length()-2)+"IBBE*");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(2);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(pattern.charAt(pattern.length()-2)+"I*E");
                t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE*");
                t.setSouthLabel(pattern.charAt(pattern.length()-2)+"I");
                t.setWestLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE");
                tiles.add(t);
                t = new Tile();
            } else {
                t.setName("<" + pattern.charAt(pattern.length()-1)+"I"+pattern.charAt(pattern.length()-2)+"IBBE*");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(2);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(pattern.charAt(pattern.length()-2)+"B*E");
                t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE*");
                t.setSouthLabel(pattern.charAt(pattern.length()-2)+"B");
                t.setWestLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE");
                tiles.add(t);
                t = new Tile();
            }
        }
        else{
            t.setName("<" + pattern.charAt(pattern.length()-1)+"I"+pattern.charAt(pattern.length()-2)+"BBBE*");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(2);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(0)+"B*E");
            t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE*");
            t.setSouthLabel(pattern.charAt(0)+"B");
            t.setWestLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE");
            tiles.add(t);
            t = new Tile();
        }
        t.setName(pattern.charAt(pattern.length()-1)+"I"+pattern.charAt(pattern.length()-1)+"IBBE*");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(1);
        t.setSouthStrength(1);
        t.setWestStrength(2);
        t.setNorthLabel(pattern.charAt(pattern.length()-1)+"IRBE");
        t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE");
        t.setSouthLabel(pattern.charAt(pattern.length()-1)+"I");
        t.setWestLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBE*");
        tiles.add(t);
        t = new Tile();
        t.setName(pattern.charAt(0)+"B"+pattern.charAt(pattern.length()-1) + "ITBC");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(2);
        t.setSouthStrength(1);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(pattern.length()-1)+"I");
        t.setEastLabel(pattern.charAt(0)+"B*EI");
        t.setSouthLabel(pattern.charAt(pattern.length()-1)+"IRBE");
        t.setWestLabel(pattern.charAt(0)+"BTBE");
        tiles.add(t);
        t = new Tile();
        
    }

    public static void getNorthTiles(ArrayList<Tile> tiles, String pattern){
        String[] modes = {"0I","1I",pattern.charAt(0)+"B"};
        Tile t = new Tile();
        //< and > tiles
        for(String c : modes){
            for(String d : modes){
                t.setName("<"+c+d + "N");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(d);
                t.setEastLabel("<"+c + "N");
                t.setSouthLabel(d);
                t.setWestLabel("<"+c + "N");
                tiles.add(t);
                t = new Tile();
                t.setName(c+d+">N");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(d);
                t.setEastLabel(c+">N");
                t.setSouthLabel(d);
                t.setWestLabel(c+">N");
                tiles.add(t);
                t = new Tile();
            }
        }
        // * tiles
        for(String c : modes){
            t.setName(c+c+"*N");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(2);
            t.setWestStrength(1);
            t.setNorthLabel(c);
            t.setEastLabel(c+">N");
            t.setSouthLabel(c+"*N");
            t.setWestLabel(c+"*N");
            tiles.add(t);
            t = new Tile();
        }
        // <* tiles
        
        String[] types = {"0", "1"};
        String[] types2 = {"B", "I"};
        for(String col : types){
            for(String colMark : types2) {
                for(String row : types){
                    for(String rowMark : types2){
                        t.setName("<" + row + rowMark + col + colMark + "*N");
                        t.setLabel(t.name);
                        t.setNorthStrength(2);
                        t.setEastStrength(1);
                        t.setSouthStrength(1);
                        t.setWestStrength(1);
                        t.setNorthLabel(col+colMark+"*N");
                        t.setEastLabel(row+rowMark+"*N");
                        t.setSouthLabel(col+colMark);
                        t.setWestLabel("<" +row+rowMark+"N");
                        tiles.add(t);
                        t = new Tile();
                    }
                }
            }
        }
        //bounding tiles
        for(String c: modes){
            //top bound
            t.setName(pattern.charAt(0)+"B"+c+"TBN");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(c);
            t.setEastLabel(pattern.charAt(0)+"BTBN");
            t.setSouthLabel(c);
            t.setWestLabel(pattern.charAt(0)+"BTBN");
            tiles.add(t);
            t = new Tile();
            //right bound
            t.setName(c + pattern.charAt(pattern.length() - 1) + "IRB>N");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(pattern.length()-1)+"IRBN");
            t.setEastLabel(c + ">N");
            t.setSouthLabel(pattern.charAt(pattern.length()-1)+"IRBN");
            t.setWestLabel(c+">" + "N");
            tiles.add(t);
            t = new Tile();
            //left bound
            t.setName("<" + c + pattern.charAt(0) + "BLBN");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(0)+"BLBN");
            t.setEastLabel("<" + c + "N");
            t.setSouthLabel(pattern.charAt(0)+"BLBN");
            t.setWestLabel("<" + c + "W");
            tiles.add(t);
            t = new Tile();

        }
        t.setName(pattern.charAt(1)+"I"+pattern.charAt(0)+"BLB*N");
        t.setLabel(t.name);
        t.setNorthStrength(2);
        t.setEastStrength(1);
        t.setSouthStrength(1);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(0)+"BLB*N");
        t.setEastLabel(pattern.charAt(1)+"I*N");
        t.setSouthLabel(pattern.charAt(0)+"BLBN");
        t.setWestLabel("<" + pattern.charAt(1)+"IW");
        tiles.add(t);
        t = new Tile();
        t.setName(pattern.charAt(0)+"B"+pattern.charAt(0)+"B*TBN");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(1);
        t.setSouthStrength(2);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(0)+"BLBN");
        t.setEastLabel(pattern.charAt(0)+"BTBN");
        t.setSouthLabel(pattern.charAt(0)+"BLB*N");
        t.setWestLabel("<" + pattern.charAt(0)+"BW");
        tiles.add(t);
        t = new Tile();
        t.setName(pattern.charAt(pattern.length()-1)+"I"+pattern.charAt(pattern.length()-1)+"I*RBN");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(1);
        t.setSouthStrength(2);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(pattern.length()-1)+"IRBN");
        t.setEastLabel(pattern.charAt(pattern.length()-1)+"I>N");
        t.setSouthLabel(pattern.charAt(pattern.length()-1)+"I*NI");
        t.setWestLabel(pattern.charAt(pattern.length()-1)+"I*N");
        tiles.add(t);
        t = new Tile();
        t.setName(pattern.charAt(0)+"B"+pattern.charAt(pattern.length()-1)+"I*RBN");
        t.setLabel(t.name);
        t.setNorthStrength(2);
        t.setEastStrength(1);
        t.setSouthStrength(1);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(pattern.length()-1)+"I*NI");
        t.setEastLabel(pattern.charAt(0)+"B>N");
        t.setSouthLabel(pattern.charAt(pattern.length()-1)+"IRBN");
        t.setWestLabel(pattern.charAt(0)+"BTBN");
        tiles.add(t);
        t = new Tile();
    }
        /*
        t.setName();
        t.setLabel();
        t.setNorthStrength();
        t.setEastStrength();
        t.setSouthStrength();
        t.setWestStrength();
        t.setNorthLabel();
        t.setEastLabel();
        t.setSouthLabel();
        t.setWestLabel();
        */
    public static void getWestTiles(ArrayList<Tile> tiles, String pattern){
        String[] modes = {"0I","1I",pattern.charAt(0)+"B"};
        Tile t = new Tile();
        //<> tiles
        //< and > tiles
        for(String c : modes){
            for(String d : modes){
                t.setName("<"+c+d + "W");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(d);
                t.setEastLabel("<"+c + "W");
                t.setSouthLabel(d);
                t.setWestLabel("<"+c + "W");
                tiles.add(t);
                t = new Tile();
                t.setName(c+d+">W");
                t.setLabel(t.name);
                t.setNorthStrength(1);
                t.setEastStrength(1);
                t.setSouthStrength(1);
                t.setWestStrength(1);
                t.setNorthLabel(d);
                t.setEastLabel(c+">W");
                t.setSouthLabel(d);
                t.setWestLabel(c+">W");
                tiles.add(t);
                t = new Tile();
            }
        }
        //* tiles
        for(String c : modes){
            t.setName(c+c+"*W");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(2);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(c+"*W");
            t.setEastLabel(c+"*W");
            t.setSouthLabel(c);
            t.setWestLabel("<"+c + "W");
            tiles.add(t);
            t = new Tile();
        }
        String[] types = {"0", "1"};
        String[] types2 = {"B", "I"};
        for(String col : types){
            for(String colMark : types2) {
                for(String row : types){
                    for(String rowMark : types2){
                        t.setName(row + rowMark + col + colMark + ">*W");
                        t.setLabel(t.name);
                        t.setNorthStrength(1);
                        t.setEastStrength(1);
                        t.setSouthStrength(1);
                        t.setWestStrength(2);
                        t.setNorthLabel(col+colMark);
                        t.setEastLabel(row+rowMark+">W");
                        t.setSouthLabel(col+colMark+"*W");
                        t.setWestLabel(row+rowMark+"*W");
                        tiles.add(t);
                        t = new Tile();
                    }
                }
            }
        }
        //bounding tiles
        for(String c : modes){
            //top bound
            t.setName(pattern.charAt(0)+"B"+c+"TBW");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(c);
            t.setEastLabel(pattern.charAt(0)+"BTBW");
            t.setSouthLabel(c);
            t.setWestLabel(pattern.charAt(0)+"BTBW");
            tiles.add(t);
            t = new Tile();
            //bottom bound
            t.setName("<" + pattern.charAt(pattern.length() - 1) + "I" + c + "BBW");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(c);
            t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBW");
            t.setSouthLabel(c);
            t.setWestLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBW");
            tiles.add(t);
            t = new Tile();
            //left bound
            t.setName("<" + c + pattern.charAt(0) + "BLBW");
            t.setLabel(t.name);
            t.setNorthStrength(1);
            t.setEastStrength(1);
            t.setSouthStrength(1);
            t.setWestStrength(1);
            t.setNorthLabel(pattern.charAt(0)+"BLBW");
            t.setEastLabel("<" + c + "W");
            t.setSouthLabel(pattern.charAt(0)+"BLBW");
            t.setWestLabel(c + ">W");
            tiles.add(t);
            t = new Tile();
        }
        //bottom bound starter
        t.setName(pattern.charAt(pattern.length()-1)+"I"+pattern.charAt(pattern.length()-1)+"I*BB");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(2);
        t.setSouthStrength(1);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(pattern.length()-1)+"I*W");
        t.setEastLabel(pattern.charAt(pattern.length()-1)+"I*BBW");
        t.setSouthLabel(pattern.charAt(pattern.length()-1)+"I");
        t.setWestLabel("<" + pattern.charAt(pattern.length()-1)+"IBBW");
        tiles.add(t);
        t = new Tile();
        //top bound extend
        t.setName(pattern.charAt(0)+"B"+pattern.charAt(1)+"I>*TBW");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(1);
        t.setSouthStrength(1);
        t.setWestStrength(2);
        t.setNorthLabel(pattern.charAt(1)+"I");
        t.setEastLabel(pattern.charAt(0)+"BTBW");
        t.setSouthLabel(pattern.charAt(1)+"I*W");
        t.setWestLabel(pattern.charAt(0)+"B*TBW");
        tiles.add(t);
        t = new Tile();
        //top bound starter
        t.setName(pattern.charAt(0)+"B"+pattern.charAt(0)+"B>*TBW");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(2);
        t.setSouthStrength(1);
        t.setWestStrength(1);
        t.setNorthLabel(pattern.charAt(0)+"B");
        t.setEastLabel(pattern.charAt(0)+"B*TBW");
        t.setSouthLabel(pattern.charAt(0)+"BLBW");
        t.setWestLabel(pattern.charAt(0)+"BTBW");
        tiles.add(t);
        t = new Tile();
        //reproduction tile
        t.setName(pattern.charAt(pattern.length()-1)+"I"+pattern.charAt(0)+"BLBCW");
        t.setLabel(t.name);
        t.setNorthStrength(1);
        t.setEastStrength(1);
        t.setSouthStrength(1);
        t.setWestStrength(2);
        t.setNorthLabel(pattern.charAt(0)+"BLBW");
        t.setEastLabel("<" + pattern.charAt((pattern.length() - 1)) + "IBBW");
        t.setSouthLabel(pattern.charAt(0)+"B");
        t.setWestLabel(pattern.charAt(pattern.length()-1)+"I*BBW");
        tiles.add(t);
        t = new Tile();
    }
    public static void colorTiles(ArrayList<Tile> tiles, boolean initValue){
        for(Tile t : tiles){
            setColor(t, initValue);
        }
    }
    public static void setColor(Tile tile, boolean initValue){
        if(tile.label.contains("0B0B"))
        {
            tile.setColor("Red");
        }
        else if(tile.label.contains("0B0I")){
            tile.setColor("Red");
        }
        else if(tile.label.contains("0B1I")){
            tile.setColor("Green");
        }
        else if(tile.label.contains("0I0B")){
            tile.setColor("Red");
        }
        else if(tile.label.contains("0I0I")){
            tile.setColor("rgb(255,0,255)");
        }
        else if(tile.label.contains("0I1I")){
            tile.setColor("Blue");
        }
        else if(tile.label.contains("1I0B")){
            tile.setColor("Black");
        }
        else if(tile.label.contains("1I0I")){
            tile.setColor("rgb(255,255,0)");
        }
        else if(tile.label.contains("1I1I")){
            tile.setColor("rgb(0,255,255)");
        }else if(tile.label.contains("1B0I")){
            tile.setColor("Black");
        } else if(tile.label.contains("1B1I")){
            tile.setColor("White");
        } else if(tile.label.contains("1B1B")){
            tile.setColor("White");
        } else if(tile.label.contains("0I1B")){
            tile.setColor("Green");
        }
        if(tile.label.contains("Init")){
            tile.setColor(initGetColor(tile.southLabel, initValue));
        }
    }    
    public static String initGetColor(String tileBottomLabel, boolean initValue){
        if(initValue){
            if(tileBottomLabel.contains("0B"))
            {
                return "Red";
            }
            else if(tileBottomLabel.contains("0I")){
                return "Red";
            }
            else {
                return "Green";
            }
        } else {
            if(tileBottomLabel.contains("0I")){
                return "Black";
            } else if(tileBottomLabel.contains("1I")){
                return "White";
            } else {
                return "White";
            }
        }
    }
    public static void markSignals(ArrayList<Tile> tiles){
        for(int i = 0; i < tiles.size(); i++){
            if(tiles.get(i).label.contains("*")){
                tiles.get(i).setColor("Red");
            } 
        }
    }
    public static void markBounds(ArrayList<Tile> tiles) {
        for(Tile tile : tiles){
            if(tile.label.contains("BB") || tile.label.contains("TB") || tile.label.contains("RB") || tile.label.contains("LB")){
                tile.setColor("Black");
            }
        }
    }
}   
