from PyTASTileSystem import *
from typing import List
import sys
import getopt
import math
from utils import get_rot_dirs, build_row, string_to_dir

def DisplayHelp():
    print("\nThis script takes as input a number, n, for the length of a counter and possibly the names")
    print("of output files, and outputs files for an aTAM system which generates a counter that counts to n")
    print("\nUsage:")
    print("-h, --help\t\t\t\tDisplays this help message.")
    print("-n <int>, --n=<int>\t\t\t Counter length .")
    print("-s <tdp file>, --seed=<tdp file>\t[Optional] TDP file to output. If not specified,")
    print("\t\t\t\t\t`<n>x<n>_square.tdp' will be used for given value n.")
    print("-t <tds file>, --tileset=<tds file>\t[Optional] TDS file to output. If not specified,")
    print("\t\t\t\t\t`<n>x<n>_square.tds' will be used for given value n.")
    print("-d <dir>, --dir=<dir>\t\t\t[Optional] Directory for output files. If not specified,")
    print("\t\t\t\t\tthe current directory will be used.")
    print("--debug\t\t\t\t[Optional] Print debug information.")
    print("")
    exit()

def BuildCounterTiles(dir: int, flip = False) -> List[PyTASTileType]:
    tile_types = []
    make_glue = lambda l, s: Glue(s, l)
    make_name = lambda l: l
    (N_, E_, S_, W_) = get_rot_dirs(dir)

    t = PyTASTileType()
    t.name = make_name('0*E')
    t.label = '0*E'
    t.sides[S_] = make_glue('l', 1)
    t.sides[W_] = make_glue('z', 1)
    t.sides[N_] = make_glue('b', 2)
    tile_types.append(t)

    t = PyTASTileType()
    t.name = make_name('0z')
    t.label = '0z'
    t.sides[S_] = make_glue('u', 1)
    t.sides[W_] = make_glue('z', 1)
    t.sides[N_] = make_glue('n', 1)
    t.sides[E_] = make_glue('z', 1)
    tile_types.append(t)

    t = PyTASTileType()
    t.name = make_name('1m')
    t.label = '1m'
    t.sides[S_] = make_glue('a', 2)
    t.sides[W_] = make_glue('c', 1)
    t.sides[N_] = make_glue('u', 1)
    t.sides[E_] = make_glue('z', 1)
    tile_types.append(t)

    t = PyTASTileType()
    t.name = make_name('0c')
    t.label = '0c'
    t.sides[S_] = make_glue('n', 1)
    t.sides[W_] = make_glue('c', 1)
    t.sides[N_] = make_glue('n', 1)
    t.sides[E_] = make_glue('c', 1)
    tile_types.append(t)

    t = PyTASTileType()
    t.name = make_name('1c')
    t.label = '1c'
    t.sides[S_] = make_glue('u', 1)
    t.sides[W_] = make_glue('c', 1)
    t.sides[N_] = make_glue('u', 1)
    t.sides[E_] = make_glue('c', 1)
    tile_types.append(t)

    t = PyTASTileType()
    t.name = make_name('1e')
    t.label = '1e'
    t.sides[S_] = make_glue('b', 2)
    t.sides[W_] = make_glue('s', 1)
    t.sides[N_] = make_glue('l', 1)
    tile_types.append(t)

    t = PyTASTileType()
    t.name = make_name('1s')
    t.label = '1s'
    t.sides[S_] = make_glue('u', 1)
    t.sides[W_] = make_glue('s', 1)
    t.sides[N_] = make_glue('u', 1)
    t.sides[E_] = make_glue('s', 1)
    tile_types.append(t)

    t = PyTASTileType()
    t.name = make_name('0*M')
    t.label = '0*M'
    t.sides[S_] = make_glue('n', 1)
    t.sides[W_] = make_glue('c', 1)
    t.sides[N_] = make_glue('a', 2)
    t.sides[E_] = make_glue('s', 1)
    tile_types.append(t)

    if flip:
        for tile in tile_types:
            glue = tile.sides[E_]
            tile.sides[E_] = tile.sides[W_]
            tile.sides[W_] = glue

    return tile_types


def CounterLengthToBitStr(length: int) -> str:
    row_len = math.ceil(math.log2(length))
    counter_value = 2**row_len - length
    bit_str = bin(counter_value)[2:]
    # Pad the beginning of the bit string with zeros
    bit_str = '0' * (row_len - len(bit_str)) + bit_str

    return bit_str


def BuildCounterSeed(dir: int, bit_str: str, debug = False, flip = False) -> List[PyTASTileType]:
    make_glue = lambda l, s: Glue(s, l)
    (N_, E_, S_, W_) = get_rot_dirs(dir)

    if not flip:
        bit_str = bit_str[::-1]

    row_len = len(bit_str)
    row_dir = E_ if flip else W_
    tiles = build_row(row_len, 'Seed', row_dir)

    if debug:
        print(f'Building seed')
        print(f'row_len: {row_len}, bit_str: {bit_str}');

    # Apply the counter bits to the top of the row
    loc = 0
    for t, bit in zip(tiles, bit_str):
        bit = int(bit)
        t.label = f'{bit}'

        allOnesBeforeMe = True
        for b in bit_str[0:loc]:
            if b == '0':
                allOnesBeforeMe = False
                break

        if loc == 0:
            if bit == 1:
                t.sides[N_] = make_glue('l', 1)
            else:
                t.sides[N_] = make_glue('b', 2)

        if loc > 0:
            if bit == 1: 
                t.sides[N_] = make_glue('u', 1)
            elif allOnesBeforeMe:
                t.sides[N_] = make_glue('a', 2)
            else:
                t.sides[N_] = make_glue('n', 1)

        if debug:
            print(f'loc: {loc}, bit: {bit}, allOnesBeforeMe: {allOnesBeforeMe}, North Glue: {t.sides[N_]}')

        loc += 1

    return tiles


def Build(dir: int, length: int, debug: bool, flip = True) -> PyTASTileSystem:
    tas = PyTASTileSystem()

    if debug: 
        print(f'Building counter for length {length} in direction {dir} flip {flip}\n')

    bit_str = CounterLengthToBitStr(length)
    seed_tiles = BuildCounterSeed(dir, bit_str, debug, flip=flip)
    counter_tiles = BuildCounterTiles(dir, flip=flip)
    seed_tile = seed_tiles[0]

    if debug:
        print('# seed tiles:', len(seed_tiles))
        print('# counter tiles:', len(counter_tiles))
        print('seed tile:', seed_tile.label)

    tas.appendTileTypes(seed_tiles)
    tas.appendTileTypes(counter_tiles)
    tas.seedPatch.append((seed_tile.name, (0, 0)))

    return tas

if __name__ == "__main__":
    try:
        # starts at the second element of argv since the first one is the script name
        # extraparams are extra arguments passed after all option/keywords are assigned
        # opts is a list containing the pair "option"/"value"
        opts, extraparams = getopt.getopt(sys.argv[1:], "hn:s:t:df:",
                            ["help", "n=", "seed=", "tileset=", "dir=", "debug", "direction=", "flip"])
    except:
        print("")
        print("Unrecognized option:" + str(sys.argv[1:]))
        DisplayHelp()
        exit()

    if len(opts) == 0:
        DisplayHelp()

    n = 16
    tdp_file = None
    tds_file = None
    directory = '.'
    direction = N
    debug = False
    flip = False

    for o,p in opts:
        if o in ['-h', '--help']:
            DisplayHelp()
        elif o in ['-n', '--n']:
            n = int(p)
        elif o in ['-d', '--dir']:
            directory = p
        elif o in ['--debug']:
            debug = True
        elif o in ['--flip', '-f']:
            flip = True
        elif o in ['--direction']:
            direction = string_to_dir(p)

        else:
            DisplayHelp()

    tas = Build(direction, length=n, debug=debug, flip=flip)
    tas.SaveToFile(f'{directory}/{n}_counter.tds')


