import os
import svgwrite
from svgwrite import cm, mm, px
from svgwrite import pt as svgPt
from reportlab.pdfgen import canvas


N = North = 0
E = East = 1
S = South = 2
W = West = 3
U = Up = 4
D = Down = 5

Dirs = [N,E,S,W]
DirAbbrevArray = ("N", "E", "S", "W", "U", "D")
DirNameArray = ("NORTH", "EAST", "SOUTH", "WEST", "UP", "DOWN")

DISPLAY_NONE  = 0
DISPLAY_LABEL = 1
DISPLAY_GLUES = 2
DISPLAY_ALL   = 3

strokeWidth = 1
svgUnits = px
skinnyFactor = 0.7
textStyle = "font-family:Arial"
defaultFillColor = 'grey'
pdfFont = "Courier"#"Helvetica"
pdfFontBold = "Helvetica-Bold"


def DirListToAbbrevStr(dir_list):
    ret_str = ''
    for d in dir_list:
        ret_str += DirAbbrevArray[d]
    return ret_str

CELLSIZE = 180

class Glue:
    """A representation of a glue on the side of a tile, including strength and label"""
    strength = 0
    label = ""
    hash = None

    def __init__(self, strength=0, label=""):
        self.strength = strength
        self.label = label

    def __eq__(self, other):
        if (isinstance(other, Glue)):
            return (self.strength == other.strength) and (self.label == other.label)
        return False

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return str(self.strength) + ":" + self.label

    def Copy(self):
        return Glue(self.strength, self.label)

    def Contains(self, string):
        return (self.label.find(string) > -1)

    def Equals(self, other):
        if (self.strength == other.strength) and (self.label == other.label):
            return True
        return False

    def Print(self, d):
        print(DirNameArray[d] + "BIND " + str(self.strength))
        print(DirNameArray[d] + "LABEL " + self.label)

    def Output(self, d):
        ret = DirNameArray[d] + "BIND " + str(self.strength) + "\n"
        ret += DirNameArray[d] + "LABEL " + self.label + "\n"
        return ret

    def Hash(self):
        if self.hash is None:
            self.hash = self.label + str(self.strength)
        return self.hash


class TileType:
    def __init__(self, name, width=100, height=100, label=None):
        self.name = name
        self.label = label
        self.color = "White"
        self.colorVector = (255, 255, 255)
        self.textColor = "Black"
        self.textColorVector = (0, 0, 0)
        self.width = width
        self.height = height
        self.dims = 2
        self.sides = {N: Glue(), E: Glue(), S: Glue(), W: Glue()}
        self.fullString = None

    def __hash__(self):
        sidesStr = ""
        for s in self.sides:
            sidesStr = sidesStr + str(self.sides[s])
        return hash(sidesStr)

    def __eq__(self, other):
        if (isinstance(other, type(self))):
            return (self.__hash__() == other.__hash__())
        return False

    def Copy(self):
        t = TileType(self.name)
        t.label = self.label
        t.color = self.color
        t.colorVector = self.colorVector
        t.textColorVector = self.textColorVector
        t.width = self.width
        t.height = self.height
        t.dims = self.dims
        t.textColor = self.textColor
        t.sides[N] = self.sides[N].Copy()
        t.sides[E] = self.sides[E].Copy()
        t.sides[S] = self.sides[S].Copy()
        t.sides[W] = self.sides[W].Copy()
        t.fullString = self.fullString
        return t

    def SetEqual(self, other):
        self.name = other.name
        self.label = other.label
        self.color = other.color
        self.colorVector = other.colorVector
        self.textColorVector = other.textColorVector
        self.width = other.width
        self.height = other.height
        self.dims = other.dims
        self.textColor = other.textColor
        self.sides[N] = other.sides[N].Copy()
        self.sides[E] = other.sides[E].Copy()
        self.sides[S] = other.sides[S].Copy()
        self.sides[W] = other.sides[W].Copy()
        self.fullString = other.fullString

    def Print(self):
        if self.IsEmpty():
            return
        if self.name == "":
            self.CreateName()
        print("TILENAME " + self.name)
        print("LABEL " + self.label)
        for i, d in self.sides.items():
            self.sides[i].Print(i)
        if len(self.color) > 0:
            print("TILECOLOR " + self.color)
        if len(self.textColor) > 0:
            print("TEXTCOLOR " + str(self.textColor))
        print("CREATE\n")

    def Output(self):
        if self.IsEmpty():
            return ""
        if self.name == "":
            self.CreateName()
        ret = "TILENAME " + self.name + "\n"
        if not self.label is None:
            ret += "LABEL " + self.label + "\n"
        for i, d in self.sides.items():
            ret += self.sides[i].Output(i)
        if (len(self.color) > 0) and (not self.color == "White"):
            ret += "TILECOLOR " + self.color + "\n"
        if (len(self.textColor) > 0) and (not self.textColor == "Black"):
            ret += "TEXTCOLOR " + str(self.textColor) + "\n"
        ret += "CREATE\n"
        return ret

    def HasMatchingGlues(self, other):
        if not len(self.sides) == len(other.sides):
            return False
        for side in self.sides:
            if not side in other.sides:
                return False
            elif self.sides[side].Equals(other.sides[side]) == False:
                return False
        return True

    def IsEmpty(self):
        for i in range(self.dims * 2):
            if not self.sides[i].strength == 0:
                return False
        return True

    def GetName(self):
        if len(self.name) == 0:
            self.CreateName()
        return self.name

    def CreateName(self):
        self.name = ""
        for i in range(self.dims * 2):
            if len(self.name) > 0:
                self.name = self.name + ","
            self.name = self.name + "[" + DirAbbrevArray[i] + "]" + str(self.sides[i].strength) + ":" + self.sides[
                i].label

    def __str__(self):
        return self.GetName()

    def ToString(self, bFull=False):
        if bFull:
            if self.fullString is None:
                self.fullString = self.GetName() + " "
                for side in self.sides:
                    self.fullString += DirAbbrevArray[side] + ":" + self.sides[side].label + "(" + str(
                        self.sides[side].strength) + ")  "
                return self.fullString
            else:
                return self.fullString
        else:
            return self.GetName()


class Tile:
    def __init__(self, ttype, coords=None):
        self.type = ttype
        self.coords = coords

    def __str__(self):
        outStr = self.type.name + str(self.coords)
        return outStr

    def __repr__(self):
        outStr = self.type.name + str(self.coords)
        return outStr
class PyTASTileType(TileType):
    """A representation of a tile type for use in the ISU TAS simulator.
       Every tile must have a unique name and a unique set of 4 glues. If
       a name is not explicitly provided, one will be formed automatically
       as a concatenation of the glues values of each side."""

    def __init__(self, name="", width=CELLSIZE, height=CELLSIZE, is3D=False, label=""):
        TileType.__init__(self, name, width, height, label)
        if is3D == False:
            self.dims = 2
            self.sides = {N: Glue(), E: Glue(), S: Glue(), W: Glue()}
        else:
            self.dims = 3
            self.sides = {N: Glue(), E: Glue(), S: Glue(), W: Glue(), U: Glue(), D: Glue()}

    def SetDimensions(self, dims):
        if self.dims == dims:
            return
        elif dims == 3:
            self.dims = 3
            self.sides[U] = Glue()
            self.sides[D] = Glue()
        elif dims == 2:
            self.dims = 2
            savedSides = self.sides
            self.sides = {N: savedSides[N], E: savedSides[E], S: savedSides[S], W: savedSides[W]}
        else:
            print("ERROR: invalid number of tile type dimensions: " + str(dims))

    def Copy(self):
        if self.dims == 2:
            t = PyTASTileType()
        else:
            t = TileType(True)
        t.sides[N] = self.sides[N].Copy()
        t.sides[E] = self.sides[E].Copy()
        t.sides[S] = self.sides[S].Copy()
        t.sides[W] = self.sides[W].Copy()
        if self.dims == 3:
            t.sides[U] = self.sides[U].Copy()
            t.sides[D] = self.sides[D].Copy()
        t.name = self.name
        t.label = self.label
        t.color = self.color
        t.textColor = self.textColor
        t.colorVector = self.colorVector
        t.textColorVector = self.textColorVector
        return t

    def MakeImage(self, mask=DISPLAY_ALL, pdfCanvas=None, svgCanvas=None, trans=None):
        tabSize = 10
        w = int(CELLSIZE-2*tabSize)
        h = int(CELLSIZE-2*tabSize)

        trans = (trans[0]*(w+(2*tabSize)), trans[1]*(h+(2*tabSize)))

        if not pdfCanvas is None:
            pdfColor = self.color#(float(self.colorVector[0]) / 255, float(self.colorVector[1]) / 255, float(self.colorVector[2]) / 255)
            #pdfCanvas.setFillColorRGB(pdfColor[0], pdfColor[1], pdfColor[2])
            pdfCanvas.setFillColor(pdfColor)
        if not svgCanvas is None:
            units = px
            svgColor = self.color#"rgb(" + str(self.colorVector[0]) + "," + str(self.colorVector[1]) + "," + str(self.colorVector[2]) + ")"
            # TODO: fix: svgGroup = svgCanvas.add(svgCanvas.g(id=self.GetName()))
            svgGroup = svgCanvas.add(svgCanvas.g())

        start_x = trans[0] + tabSize
        start_y = trans[1] + tabSize
        if not pdfCanvas is None:
            pdfCanvas.rect(start_x, start_y, w, h, stroke=1, fill=1)
        if not svgCanvas is None:
            svgGroup.add(svgCanvas.rect(insert=(start_x, start_y), size=(w, h),
                                        fill=svgColor, stroke='black', stroke_width=5))

        if not pdfCanvas is None:
            pdfCanvas.setFillColor("black")

        # Draw northern tabs
        numTabs = self.sides[N].strength
        if numTabs > 0:
            spacing = w / (numTabs + 1)
            currLoc = spacing
            for n in range(numTabs):
                if not pdfCanvas is None:
                    pdfCanvas.rect(trans[0] + currLoc - (tabSize / 2), trans[1] + h + tabSize, tabSize, tabSize, stroke=1, fill=1)
                if not svgCanvas is None:
                    svgGroup.add(svgCanvas.rect(insert=(trans[0] + currLoc - (tabSize / 2), trans[1]),
                                                size=(tabSize, tabSize), fill='black', stroke='black', stroke_width=0))
                currLoc = currLoc + spacing

        # Draw southern tabs
        numTabs = self.sides[S].strength
        if numTabs > 0:
            spacing = w / (numTabs + 1)
            currLoc = spacing
            for n in range(numTabs):
                if not pdfCanvas is None:
                    pdfCanvas.rect(trans[0] + currLoc - (tabSize / 2), trans[1], tabSize, tabSize, stroke=1, fill=1)
                if not svgCanvas is None:
                    svgGroup.add(svgCanvas.rect(insert=(trans[0] + currLoc - (tabSize / 2), trans[1] + h + tabSize),
                                                size=(tabSize, tabSize), fill='black', stroke='black', stroke_width=0))
                currLoc = currLoc + spacing

        # Draw western tabs
        numTabs = self.sides[W].strength
        if numTabs > 0:
            spacing = h / (numTabs + 1)
            currLoc = spacing
            for n in range(numTabs):
                if not pdfCanvas is None:
                    pdfCanvas.rect(trans[0], trans[1] + currLoc - (tabSize / 2), tabSize, tabSize, stroke=1, fill=1)
                if not svgCanvas is None:
                    svgGroup.add(svgCanvas.rect(insert=(trans[0], trans[1] + currLoc - (tabSize / 2)),
                                                size=(tabSize, tabSize), fill='black', stroke='black', stroke_width=0))
                currLoc = currLoc + spacing

        # Draw eastern tabs
        numTabs = self.sides[E].strength
        if numTabs > 0:
            spacing = h / (numTabs + 1)
            currLoc = spacing
            for n in range(numTabs):
                if not pdfCanvas is None:
                    pdfCanvas.rect(trans[0] + w + tabSize, trans[1] + currLoc - (tabSize / 2), tabSize, tabSize, stroke=1, fill=1)
                if not svgCanvas is None:
                    svgGroup.add(svgCanvas.rect(insert=(trans[0] + w + tabSize, trans[1] + currLoc - (tabSize / 2)),
                                                size=(tabSize, tabSize), fill='black', stroke='black', stroke_width=0))
                currLoc = currLoc + spacing

        if not pdfCanvas is None:
            pdfColor = (float(self.textColorVector[0]) / 255, float(self.textColorVector[1]) / 255,
                        float(self.textColorVector[2]) / 255)
        if not svgCanvas is None:
            svgColor = "rgb(" + str(self.textColorVector[0]) + "," + str(self.textColorVector[1]) + "," + str(
                self.textColorVector[2]) + ")"

        # Draw tile label
        wLabel = w / 2
        hLabel = h / 2
        defPixHt = 30
        glueLabelScale = 1.0
        fname = "Helvetica"  # "Arial Narrow"
        pdfFont = "Helvetica"
        pdfFontBold = "Helvetica-Bold"

        # font = wx.Font(pointSize = 30, family = wx.DEFAULT, style = wx.NORMAL, weight = wx.NORMAL, faceName = 'Arial')
        if (not self.label is None) and ((mask == DISPLAY_ALL or mask == DISPLAY_LABEL)):

            stroke_color = 'white' if self.color in ['black', 'Black', 'blue', 'Blue'] else 'black'
            text = self.label
            if len(text) > 0:
                if len(text) < 3:
                    text = f' {text} '
                textLen = len(text)
                textSize = w / textLen
                x_start = trans[0] + (w / 2) - (wLabel / 2)
                y_start = trans[1] + (h / 2) + textSize
                if not pdfCanvas is None:
                    textobject = pdfCanvas.beginText(x_start, y_start - textSize)
                    textobject.setFont(pdfFontBold, textSize)
                    #textobject.setFillColorRGB(0, 0, 0)#pdfColor[0], pdfColor[1], pdfColor[2])
                    textobject.setFillColor(stroke_color)
                    textobject.textOut(text)
                    pdfCanvas.drawText(textobject)
                if not svgCanvas is None:
                    textLength = w * 0.6
                    svg_text = svgCanvas.text(text,
                                              insert=(x_start, y_start),
                                              textLength=textLength*svgUnits, font_size=textSize*svgUnits,
                                              style="font-weight:bold;font-family:" + fname,
                                              stroke=stroke_color, fill=stroke_color)
                    svgGroup.add(svg_text)

        if (mask == DISPLAY_ALL) or (mask == DISPLAY_GLUES):
            longestLabel = ""
            for d in Dirs:
                if len(self.sides[d].label) > len(longestLabel):
                    longestLabel = self.sides[d].label

            defWLabel = (w - 6 * tabSize) * glueLabelScale
            defHLabel = (h - 6 * tabSize) * glueLabelScale
            wLabel = defWLabel
            hLabel = defHLabel

            # if len(self.sides[N].label) > 0:
            #     # Draw north glue label
            #     wLabel = defWLabel
            #     hLabel = defHLabel
            #
            #     if not pdfCanvas is None:
            #         textobject = pdfCanvas.beginText((w / 2) - 0.8 * (wLabel / 2), h - hLabel)
            #         textobject.setFont(pdfFont, pdfFontSize)
            #         textobject.setFillColorRGB(pdfColor[0], pdfColor[1], pdfColor[2])
            #         textobject.textOut(self.sides[N].label)
            #         pdfCanvas.drawText(textobject)
            #     if not svgCanvas is None:
            #         text = svgCanvas.text(self.sides[N].label,
            #                               insert=(trans[0] + (w / 2) - (wLabel / 2), trans[1] + 1.3 * hLabel),
            #                               font_size=font.GetPointSize() * svgPt,
            #                               style="font-family:" + fname)  # ,textLength=textSize*units)
            #         svgGroup.add(text)
            #
            # if len(self.sides[S].label) > 0:
            #     # Draw south glue label
            #     wLabel = defWLabel
            #     hLabel = defHLabel
            #
            #     if not pdfCanvas is None:
            #         textobject = pdfCanvas.beginText((w / 2) - 0.8 * (wLabel / 2), pdfFontSize)
            #         textobject.setFont(pdfFont, pdfFontSize)
            #         textobject.setFillColorRGB(pdfColor[0], pdfColor[1], pdfColor[2])
            #         textobject.textOut(self.sides[S].label)
            #         pdfCanvas.drawText(textobject)
            #     if not svgCanvas is None:
            #         text = svgCanvas.text(self.sides[S].label,
            #                               insert=(trans[0] + (w / 2) - (wLabel / 2), trans[1] + h - 1.3 * hLabel / 2),
            #                               font_size=font.GetPointSize() * svgPt,
            #                               style="font-family:" + fname)  # ,textLength=textSize*units)
            #         svgGroup.add(text)
            #
            # if len(self.sides[W].label) > 0:
            #     # Draw west glue label
            #     wLabel = defWLabel
            #     hLabel = defHLabel
            #
            #     if not pdfCanvas is None:
            #         textobject = pdfCanvas.beginText((h / 2) - 0.8 * (wLabel / 2), -(tabSize + pdfFontSize))
            #         textobject.setFont(pdfFont, pdfFontSize)
            #         textobject.setFillColorRGB(pdfColor[0], pdfColor[1], pdfColor[2])
            #         pdfCanvas.rotate(90)
            #         textobject.textOut(self.sides[W].label)
            #         pdfCanvas.drawText(textobject)
            #         pdfCanvas.rotate(-90)
            #     if not svgCanvas is None:
            #         text = svgCanvas.text(self.sides[W].label,
            #                               insert=(-(trans[1] + (h / 2) + (wLabel / 2)), trans[0] + hLabel + tabSize),
            #                               font_size=font.GetPointSize() * svgPt,
            #                               style="font-family:" + fname)  # ,textLength=textSize*units)
            #         text.rotate(270)
            #         svgGroup.add(text)
            #
            # if len(self.sides[E].label) > 0:
            #     # Draw east glue label
            #     wLabel = defWLabel
            #     hLabel = defHLabel
            #
            #     if not pdfCanvas is None:
            #         textobject = pdfCanvas.beginText(-(h / 2) - 0.8 * (wLabel / 2), w - (tabSize + pdfFontSize))
            #         textobject.setFont(pdfFont, pdfFontSize)
            #         textobject.setFillColorRGB(pdfColor[0], pdfColor[1], pdfColor[2])
            #         pdfCanvas.rotate(-90)
            #         textobject.textOut(self.sides[E].label)
            #         pdfCanvas.drawText(textobject)
            #         pdfCanvas.rotate(90)
            #     if not svgCanvas is None:
            #         text = svgCanvas.text(self.sides[E].label, insert=(
            #         (trans[1] + (h / 2) - (wLabel / 2)), -(trans[0] + w - tabSize - hLabel)),
            #                               font_size=font.GetPointSize() * svgPt,
            #                               style="font-family:" + fname)  # ,textLength=textSize*units)
            #         text.rotate(90)
            #         svgGroup.add(text)

    def SetEqual(self, other):
        TileType.SetEqual(self, other)

    def SetInputSides(self, inputSet):
        for k in inputSet:
            self.sides[k] = Glue(inputSet[k].strength, inputSet[k].label)


class PyTASTileSystem:

    def __init__(self):
        self.assembly = None
        self.tileTypes = dict()
        self.seedPatch = []
        self.temperature = 2  # Default temperature = 2
        self.bHas3D = False
        self.systemDefFile = None
        self.useMacrotileMode = False

    def appendTileType(self, tileType: PyTASTileType):
        self.tileTypes[tileType.name] = tileType

    def appendTileTypes(self, tileTypes: list[PyTASTileType]):
        for tileType in tileTypes:
            self.appendTileType(tileType)

    def AssemblyFromString(self, assemblyString):
        assembly = []
        lines = assemblyString.split("\n")
        for line in lines:
            line = line.strip()
            if len(line) > 0:
                (seedTileName, (seedTileXLoc, seedTileYLoc, seedTileZLoc)) = self.SeedTileFromLine(line)
                assembly.append((seedTileName, (seedTileXLoc, seedTileYLoc, seedTileZLoc)))
        return assembly

    def SeedTileFromLine(self, line):
        seedLocArr = line.split(" ")
        seedTileName = seedLocArr[0]
        seedTileXLoc = int(seedLocArr[1])
        seedTileYLoc = int(seedLocArr[2])
        seedTileZLoc = 0
        if len(seedLocArr) > 3:
            seedTileZLoc = int(seedLocArr[3])
        return (seedTileName, (seedTileXLoc, seedTileYLoc, seedTileZLoc))

    def SaveToFile(self, base_file_name):
        if len(base_file_name) < 1:
            return False

        tdp_file_name = base_file_name + '.tdp'
        #TODO: strip directory names off if they exist
        tds_file_name = base_file_name + '.tds'
        tds_base_name = os.path.basename(tds_file_name)
        if os.path.isabs(tdp_file_name) == False:
            tdp_file_name = "./" + tdp_file_name
            tdp_file_name = os.path.abspath(tdp_file_name)

        with open(tdp_file_name, 'w') as tdp_file:
            tdp_file.write(tds_base_name + '\n')
            tdp_file.write(f'Temperature={self.temperature}\n')
            for seed_entry in self.seedPatch:
                x = seed_entry[1][0]
                y = seed_entry[1][1]
                z = 0 if len(seed_entry[1]) < 3 else seed_entry[1][2]
                tdp_file.write(f'{seed_entry[0]} {x} {y} {z}\n')

        if os.path.isabs(tds_file_name) == False:
            tds_file_name = "./" + tds_file_name
            tds_file_name = os.path.abspath(tds_file_name)

        with open(tds_file_name, 'w') as tds_file:
            for tile_type_name in self.tileTypes:
                tile_type = self.tileTypes[tile_type_name]
                tds_file.write(tile_type.Output() + '\n')
        return True

    def LoadFromFile(self, fileName, backgroundColor=None, bLoadTileset=True):
        if len(fileName) < 1:
            return False
        elif os.path.isabs(fileName) == False:
            fileName = "./" + fileName
            fileName = os.path.abspath(fileName)

        if bLoadTileset == True:
            self.tileTypes = dict()
        self.seedPatch = []
        self.temperature = 2  # Default temperature = 2
        self.bHas3D = False

        # try:
        os.stat(fileName)
        rFile = open(fileName, "r")
        print("Using tile system definition file '" + fileName + "'")
        lines = rFile.readlines()
        rFile.close()

        self.systemDefFile = fileName

        tdsFileName = None
        if bLoadTileset == True:
            tdsFileName = lines[0].strip()
            if os.path.isabs(tdsFileName) == False:
                tdsFileName = os.path.dirname(fileName) + "/" + tdsFileName
                tdsFileName = os.path.abspath(tdsFileName)
            tilesetLoadRet = self.LoadTileSet(tdsFileName, backgroundColor=backgroundColor)
            if not tilesetLoadRet == True:
                return tilesetLoadRet

        for line in lines[1:]:
            if line.startswith("Temperature="):
                temperatureStr = line[len("Temperature="):]
                self.temperature = int(temperatureStr)
            elif line.startswith("Mode="):
                modeStr = line[len("Mode="):]
            elif line.startswith("MacrotileSize="):
                tempStr = line[len("MacrotileSize="):]
                tempList = tempStr.split(",")
                self.macrotileWidth = int(tempList[0])
                self.macrotileHeight = int(tempList[1])
                self.useMacrotileMode = True
                if self.macrotileCenter is None:
                    self.macrotileCenter = (int(self.macrotileWidth / 2), int(self.macrotileHeight / 2))
            elif line.startswith("MacrotileCenter="):
                tempStr = line[len("MacrotileCenter="):]
                try:
                    tempList = tempStr.split(",")
                    xCoord = int(tempList[0])
                    yCoord = int(tempList[1])
                    self.macrotileCenter = (xCoord, yCoord)
                except:
                    self.macrotileCenter = (0, 0)
                    if not self.macrotileWidth is None:
                        self.macrotileCenter = (int(self.macrotileWidth / 2), int(self.macrotileHeight / 2))
                    print("ERROR: MacrotileCenter must be specified as coords `MacrotileCenter=x,y'. Invalid line was `" + line + "'")
            elif line.startswith("MacrotileImageDir="):
                self.macrotileImageDir = line[len("MacrotileImageDir="):].strip()
                if os.path.isabs(self.macrotileImageDir) == False:
                    self.macrotileImageDir = os.path.dirname(fileName) + "/" + self.macrotileImageDir
                    self.macrotileImageDir = os.path.abspath(self.macrotileImageDir)

            else:
                (seedTileName, (seedTileXLoc, seedTileYLoc, seedTileZLoc)) = self.SeedTileFromLine(line)
                if (self.bHas3D == True) or (not seedTileZLoc == 0):
                    self.bHas3D = True
                    self.seedPatch.append((seedTileName, (seedTileXLoc, seedTileYLoc, seedTileZLoc)))
                else:
                    self.seedPatch.append((seedTileName, (seedTileXLoc, seedTileYLoc)))

        if self.useMacrotileMode == True:
            print(f"Using macrotile mode, with macrotile size {(self.macrotileWidth, self.macrotileHeight)} and center {self.macrotileCenter}")

        self.tileSystemDef = dict()
        if bLoadTileset == True:
            self.tileSystemDef['tdsFile'] = tdsFileName
        self.tileSystemDef['temperature'] = self.temperature

        # If seed had 3D coords, make sure all tile types are 3D
        if self.bHas3D == True:
            for ttName in self.tileTypes:
                self.tileTypes[ttName].SetDimensions(3)

        return True


    def LoadTileSet(self, tdsFileName, backgroundColor=None):
        print("Reading tile set from '" + tdsFileName + "'")
        os.stat(tdsFileName)
        rFile = open(tdsFileName, "r")
        lines = rFile.readlines()
        rFile.close()

        i = 0
        ttype = PyTASTileType()
        while i < len(lines):
            lineArr = lines[i].strip().split(" ")
            if lineArr[0] == "TILENAME":
                ttype.name = lineArr[1]
            elif lineArr[0] == "LABEL" and len(lineArr) > 1:
                ttype.label = lineArr[1]
            elif lineArr[0] == "TILECOLOR" and len(lineArr) > 1:
                ttype.color = lines[i][len("TILECOLOR "):].strip()
            elif lineArr[0] == "TEXTCOLOR" and len(lineArr) > 1:
                ttype.textColor = lines[i][len("TEXTCOLOR "):].strip()
            elif lineArr[0] == "NORTHLABEL" and len(lineArr) > 1:
                ttype.sides[N].label = lineArr[1]
                if ttype.sides[N].label == "-99":
                    ttype.sides[N].strength = 0
                    ttype.sides[N].label = ""
            elif lineArr[0] == "EASTLABEL" and len(lineArr) > 1:
                ttype.sides[E].label = lineArr[1]
                if ttype.sides[E].label == "-99":
                    ttype.sides[E].strength = 0
                    ttype.sides[E].label = ""
            elif lineArr[0] == "SOUTHLABEL" and len(lineArr) > 1:
                ttype.sides[S].label = lineArr[1]
                if ttype.sides[S].label == "-99":
                    ttype.sides[S].strength = 0
                    ttype.sides[S].label = ""
            elif lineArr[0] == "WESTLABEL" and len(lineArr) > 1:
                ttype.sides[W].label = lineArr[1]
                if ttype.sides[W].label == "-99":
                    ttype.sides[W].strength = 0
                    ttype.sides[W].label = ""
            elif lineArr[0] == "UPLABEL" and len(lineArr) > 1:
                self.bHas3D = True
                ttype.SetDimensions(3)
                ttype.sides[U].label = lineArr[1]
                if ttype.sides[U].label == "-99":
                    ttype.sides[U].strength = 0
                    ttype.sides[U].label = ""
            elif lineArr[0] == "DOWNLABEL" and len(lineArr) > 1:
                self.bHas3D = True
                ttype.SetDimensions(3)
                ttype.sides[D].label = lineArr[1]
                if ttype.sides[D].label == "-99":
                    ttype.sides[D].strength = 0
                    ttype.sides[D].label = ""
            elif lineArr[0] == "NORTHBIND" and len(lineArr) > 1:
                ttype.sides[N].strength = int(lineArr[1])
            elif lineArr[0] == "EASTBIND" and len(lineArr) > 1:
                ttype.sides[E].strength = int(lineArr[1])
            elif lineArr[0] == "SOUTHBIND" and len(lineArr) > 1:
                ttype.sides[S].strength = int(lineArr[1])
            elif lineArr[0] == "WESTBIND" and len(lineArr) > 1:
                ttype.sides[W].strength = int(lineArr[1])
            elif lineArr[0] == "UPBIND" and len(lineArr) > 1:
                self.bHas3D = True
                ttype.SetDimensions(3)
                ttype.sides[U].strength = int(lineArr[1])
            elif lineArr[0] == "DOWNBIND" and len(lineArr) > 1:
                self.bHas3D = True
                ttype.SetDimensions(3)
                ttype.sides[D].strength = int(lineArr[1])
            elif lineArr[0] == "CREATE":
                if self.tileTypes.get(ttype.name) == None:
                    self.tileTypes[ttype.name] = ttype
                elif self.tileTypes.get(ttype.name) != None:
                    print("Warning, Duplicate tile name: " + ttype.name + "\tThis may cause unexpected behavior")
                    duplicateNum = 1
                    while self.tileTypes.get(ttype.name + "(" + str(duplicateNum) + ")") != None:
                        duplicateNum += 1
                    ttype.name = ttype.name + "(" + str(duplicateNum) + ")"
                    self.tileTypes[ttype.name] = ttype
                ttype = PyTASTileType()
            i = i + 1

        if self.bHas3D == True:
            for ttName in self.tileTypes:
                self.tileTypes[ttName].SetDimensions(3)

        self.tilesetDefFile = tdsFileName

        # MJP: This greatly slows the loading of large tile sets, but increases
        #      the initial speed of assembly until all tile type images have been
        #      created and cached
        # self.CreateTileTypeImages(backgroundColor=backgroundColor)

        print("Loaded tile set with size: " + str(len(self.tileTypes)))

        return True
