from PIL import Image
import math
import getopt
import sys
import os
import random
from PyTASTileSystem import *

OPP_DIRS = {N:S, E:W, S:N, W:E}



def DisplayHelp():
    print("\nThis script takes as input a path to an image file and outputs an aTAM")
    print("of system which weakly self-assembles the image in black and white. It also")
    print("outputs a black and white bitmap of the image.")
    print("\nUsage:")
    print("-h, --help\t\t\t\tDisplays this help message.")
    print("-i <image>, --image=<image>\t\tName of the image file to be used as input.")
    print("-o <out>, --out=<out>\t\t\t[Optional] Base name of the output files. If not specified,")
    print("\t\t\t\t\t the base name of the input file is used.")
    print("-r, --random\t\t\t\tUse a randomly generated set of black and white pixels.")
    print("-t <threshold>, --thresh=<threshold>\t[Optional] Integer from 0-255 that specifies,")
    print("\t\t\t\t\t the minimum pixel value to convert to white. Defaults to 200.")
    print("-d <dim>, --dim=<dim>\t\t\t[Optional] Maximum dimension of output image in pixels.")
    print("\t\t\t\t\t Must be less than or equal to maximum dimension of input image.")
    print("-c, --coop\t\t\t\t[Optional] If specified, the filler tiles will use cooperative attachment.")
    print("")
    exit()



def get_new_pixel(pixel, threshold):
    # if looks like black, convert to black
    #if (pixel[0] <= threshold) and (pixel[1] <= threshold) and (pixel[2] <= threshold):
    if (pixel[0] > threshold) and (pixel[1] > threshold) and (pixel[2] > threshold):
        new_pixel = (255, 255, 255)
    # if looks like white, convert to white
    else:
        new_pixel = (0, 0, 0)
    return new_pixel

def get_pixel_color(pixel):
    if pixel[0] > 0:
        return ''
    else:
        return 'Black'

def get_tile_type_name(coords):
    return f'({coords[0]},{coords[1]})'

def get_row_colors_pattern(start_x, y, height, row_width, direction, img, threshold):
    row_colors_str = ''

    for i in range(row_width):
        if direction == E:
            row_x = start_x + i
        else:
            row_x = start_x - i
        if row_x < img.size[0]:
            pixel_loc = (row_x, height-y-1)
            row_pixel = img.getpixel(pixel_loc)
            row_pixel_color = get_pixel_color(get_new_pixel(row_pixel, threshold))
            if row_pixel_color == 'Black':
                row_colors_str += '0'
            else:
                row_colors_str += '1'
    return row_colors_str

def get_tile_types_for_pattern(row_colors_str, direction, b_name_hack=False, b_use_coop=False):
    row_tile_types = dict()

    for i in range(len(row_colors_str)):
        glue_strength = 2
        if b_use_coop:
            glue_strength = 1
        name_add = ''
        if b_name_hack and (direction == W):
            name_add = 'W'
        row_tile_type_name = f'{row_colors_str[i:]}{name_add}'
        row_tile_type = PyTASTileType(name=row_tile_type_name, label=row_tile_type_name)
        row_tile_type.sides[OPP_DIRS[direction]] = Glue(glue_strength, row_tile_type_name)
        if i < (len(row_colors_str)-1):
            row_tile_type.sides[direction] = Glue(glue_strength, f'{row_colors_str[i+1:]}{name_add}')
        if b_use_coop:
            row_tile_type.sides[N] = Glue(glue_strength, 'x')
            row_tile_type.sides[S] = Glue(glue_strength, 'x')
        row_tile_type.color = 'Black' if row_colors_str[i] == '0' else ''
        row_tile_types[row_tile_type_name] = row_tile_type
    return row_tile_types

def get_random_image(max_dim):
    retImg = Image.new('RGB', (max_dim, max_dim))
    for x in range(max_dim):
        for y in range(max_dim):
            pix_val = 0 if random.randint(0,1) == 0 else 255
            new_pixel = (pix_val, pix_val, pix_val)
            retImg.putpixel((x,y), new_pixel)
    return retImg
    
def get_transformed_image(img, threshold, max_dim=None):
    width = img.size[0]
    height = img.size[1]
    print(f'Input image size: {img.size}')

    if max_dim is None:
        new_width = width
        new_height = height
        new_size = (new_width, new_height)
        n = 1
    else:
        if width > height:
            n = int(width / max_dim)
        else:
            n = int(height / max_dim)
        new_width = int(width / n)
        new_height = int(height / n)
        new_size = (new_width, new_height)
        print(f'Output image size will be: {new_size} -> each {n}x{n} square of pixels will be converted to one pixel')

    newImg = Image.new(img.mode, new_size)

    for x in range(new_width):
        for y in range(new_height):
            pixel_sum = (0, 0, 0)
            pixel_count = 0
            for i in range(n+1):
                for j in range(n+1):
                    pix_x = (x * n) + i
                    pix_y = (y * n) + i
                    if (pix_x < width) and (pix_y < height):
                        pixel = img.getpixel((pix_x, pix_y))
                        pixel_sum = (pixel_sum[0] + pixel[0], pixel_sum[1] + pixel[1], pixel_sum[2] + pixel[2])
                        pixel_count += 1
            pixel_ave = (int(pixel_sum[0] / pixel_count), int(pixel_sum[1] / pixel_count), int(pixel_sum[2] / pixel_count))
            new_pixel = get_new_pixel(pixel_ave, threshold)
            newImg.putpixel((x,y), new_pixel)

    return newImg
    

def get_tiles_for_column(x, img, threshold, left_distance, right_distance, full_row_distance, tas, b_rotation_supported=False, b_use_coop=False):
    height = img.size[1]

    truncation_amount = full_row_distance - left_distance
    if (left_distance > 0) and (left_distance < full_row_distance):
        # Make dictionary of truncated values
        pass

    for y in range(1, height):
        pixel = img.getpixel((x,height-y-1))
        new_pixel = get_new_pixel(pixel, threshold)
        tile_type_name = get_tile_type_name((x,y))
        tile_type = PyTASTileType(name=tile_type_name, label=tile_type_name)
        tile_type.sides[S] = Glue(2, tile_type_name)
        north_nbr = get_tile_type_name((x,y+1))
        tile_type.sides[N] = Glue(2, north_nbr)
        tile_type.color = get_pixel_color(new_pixel)

        glue_strength = 2
        if b_use_coop:
            glue_strength = 1
            
        # Compute hard-coded row to the east
        if right_distance > 0:
            row_colors_str = get_row_colors_pattern(x+1, y, height, right_distance, E, img, threshold)
            if row_colors_str is not None:
                tile_type.sides[E] = Glue(glue_strength, f'{row_colors_str}')
                
            if (row_colors_str is not None):
                # Make the hard-coded tiles for this pattern
                row_tile_types = get_tile_types_for_pattern(row_colors_str, E, b_use_coop=b_use_coop)
                tas.tileTypes.update(row_tile_types)

        if left_distance > 0:
            # Compute hard-coded row to the west
            row_colors_str = get_row_colors_pattern(x-1, y, height, left_distance, W, img, threshold)

            if b_rotation_supported:
                #TODO: set west glue to use east facing tiles
                pass
            else:

                if row_colors_str is not None:
                    tile_type.sides[W] = Glue(glue_strength, f'{row_colors_str}W')

                if (row_colors_str is not None):
                    # Create hard-coded tiles to grow west
                    row_tile_types = get_tile_types_for_pattern(row_colors_str, W, True, b_use_coop=b_use_coop)
                    tas.tileTypes.update(row_tile_types)

        tas.tileTypes[tile_type_name] = tile_type


def get_image_tas(img, threshold, width_multiplier, b_use_coop):
    width = img.size[0]
    height = img.size[1]
    print(f'Making tile set for image size: {img.size}')

    hard_coded_row_widths = width_multiplier * math.floor(math.log(width, 2))
    column_spacing = (hard_coded_row_widths * 2) + 1
    num_cols = math.floor(width / column_spacing)
    print(f'num_cols: {num_cols}, hard_coded_row_widths: {hard_coded_row_widths}')

    tas = PyTASTileSystem()

    # Determine if there will be an incomplete column + left/right row group
    last_column = None
    last_column_left_distance = 0
    last_column_right_distance = 0
    leftover = width % column_spacing
    if leftover > 0:
        if leftover <= hard_coded_row_widths:
            last_column = width - 1
            last_column_left_distance = leftover - 1
        elif leftover > (hard_coded_row_widths + 1):
            last_column_right_distance = leftover - (hard_coded_row_widths + 1)
    #print(f'leftover: {leftover}, last_column_left_distance: {last_column_left_distance}, last_column_right_distance: {last_column_right_distance}')
    
    # Make seed and first row of tiles
    seed_tile_type_name = None
    start_x = hard_coded_row_widths
    if b_use_coop:
        start_x = 0
    for x in range(start_x, width-last_column_right_distance):
        pixel = img.getpixel((x,height-1))
        new_pixel = get_new_pixel(pixel, threshold)
        tile_type_name = get_tile_type_name((x,0))
        tile_type = PyTASTileType(name=tile_type_name, label=tile_type_name)
        if x == start_x:
            seed_tile_type_name = tile_type_name
        else:
            west_nbr = get_tile_type_name((x-1,0))
            tile_type.sides[W] = Glue(2, west_nbr)
        if x < (width-1):
            tile_type.sides[E] = Glue(2, tile_type_name)

        north_nbr = get_tile_type_name((x,1))
        if ((x + hard_coded_row_widths + 1) % column_spacing) == 0:
            tile_type.sides[N] = Glue(2, north_nbr)
        elif (x == (width - 1)) and (last_column_left_distance > 0):
            tile_type.sides[N] = Glue(2, north_nbr)
        elif b_use_coop:
            tile_type.sides[N] = Glue(1, 'x')
        tile_type.color = get_pixel_color(new_pixel)
        tas.tileTypes[tile_type_name] = tile_type

    tas.seedPatch.append((seed_tile_type_name, (hard_coded_row_widths, 0)))

    # Compute hard-coded row to the west of the seed and at east end of seed row
    if not b_use_coop:
        row_colors_str = get_row_colors_pattern(hard_coded_row_widths-1, 0, height, hard_coded_row_widths, W, img, threshold)
        tas.tileTypes[seed_tile_type_name].sides[W] = Glue(2, f'{row_colors_str}W')
        row_tile_types = get_tile_types_for_pattern(row_colors_str, W, True, b_use_coop=b_use_coop)
        tas.tileTypes.update(row_tile_types)

        row_colors_str = get_row_colors_pattern(width-last_column_right_distance, 0, height, last_column_right_distance, E, img, threshold)
        tas.tileTypes[f'({width-last_column_right_distance-1},0)'].sides[E] = Glue(2, f'{row_colors_str}')
        row_tile_types = get_tile_types_for_pattern(row_colors_str, E, True, b_use_coop=b_use_coop)
        tas.tileTypes.update(row_tile_types)

    
    # Get tiles for hard-coded columns and hard-coded row segments
    for x in range(width):
        if ((x + hard_coded_row_widths + 1) % column_spacing) == 0:
            get_tiles_for_column(x, img, threshold, hard_coded_row_widths, hard_coded_row_widths, hard_coded_row_widths, tas, b_rotation_supported, b_use_coop)
        if x == (width - 1) and (last_column_left_distance > 0):
            get_tiles_for_column(x, img, threshold, last_column_left_distance, last_column_right_distance, hard_coded_row_widths, tas, b_rotation_supported, b_use_coop)

    return tas


if __name__ == "__main__":
    try:
        # starts at the second element of argv since the first one is the script name
        # extraparams are extra arguments passed after all option/keywords are assigned
        # opts is a list containing the pair "option"/"value"
        opts, extraparams = getopt.getopt(sys.argv[1:], "hi:o:t:d:cr",
                            ["help", "image=", "out=", "thresh=", "dim=", "coop", "random"])
        #print 'Opts:',opts
        #print 'Extra parameters:',extraparams
    except:
        print("")
        print("Unrecognized option:" + str(sys.argv[1:]))
        DisplayHelp()

    if len(opts) == 0:
        DisplayHelp()


    image_file_name = None
    b_use_random = False
    output_file_base_name = None
    threshold = 200
    max_dim = None
    b_rotation_supported = False
    b_use_coop = False

    for o,p in opts:
        if o in ['-h', '--help']:
            DisplayHelp()
        elif o in ['-i', '--image']:
            image_file_name = p
        elif o in ['-o', '--out']:
            output_file_base_name = p
        elif o in ['-t', '--thresh']:
            threshold = int(p)
            if (threshold < 0) or (threshold > 255):
                DisplayHelp()
        elif o in ['-d', '--dim']:
            max_dim = int(p)
        elif o in ['-c', '--coop']:
            b_use_coop = True
        elif o in ['-r', '--random']:
            b_use_random = True
        else:
            DisplayHelp()

    if (image_file_name is None) and (b_use_random == False):
        print(f'\nERROR: No input file specified.')
        DisplayHelp()
    elif b_use_random:
        if max_dim is None:
            print(f'\nERROR: Maximum dimension of output image must be specified if a random image is being generated.')
            DisplayHelp()
        if output_file_base_name is None:
            output_file_base_name = f'random_{max_dim}x{max_dim}'

        newImg = get_random_image(max_dim)
    else:
        img = Image.open(image_file_name)

        if not max_dim is None:
            img_width = img.size[0]
            img_height = img.size[1]
            curr_max_dim = img_width if img_width >= img_height else img_height
            if max_dim > curr_max_dim:
                print(f'\nERROR: The input image has dimensions {(img_width, img_height)}. If specified\nthe maximum allowed dimension must be less than or equal to {curr_max_dim}.')
                DisplayHelp()
    
        if output_file_base_name is None:
            output_file_base_name = os.path.splitext(image_file_name)[0]

        newImg = get_transformed_image(img, threshold, max_dim)

    output_image_file_name = f'{output_file_base_name}.bmp'
    newImg.save(output_image_file_name)
    print(f"Output black and white bitmap to '{output_image_file_name}'")

    width_multiplier = 1
    tas = None
    last_tas = None
    while (last_tas is None) or (len(tas.tileTypes) < len(last_tas.tileTypes)):
        if not tas is None:
            last_tas = tas
        tas = get_image_tas(newImg, threshold, width_multiplier, b_use_coop)
        print(f'For width_multiplier {width_multiplier}, got TAS with {len(tas.tileTypes)} tile types')
        width_multiplier += 1
    tas = last_tas

    num_pixels = newImg.size[0] * newImg.size[1]
    big_dim = max(newImg.size[0], newImg.size[1])
    optimal_size = math.ceil(num_pixels / math.log(big_dim))
    
    tas.SaveToFile(output_file_base_name)
    print(f"Output TAS to files '{output_file_base_name}' with {len(tas.tileTypes)} tile types for image with {newImg.size[0] * newImg.size[1]} pixels")
    #print(f'Optimal tile set size is approximately {optimal_size}')

